@@ -17,42 +17,11 @@
 """This module manages the artifacts produced by conversion passes."""
 
 
-import os
-
 from cvs2svn_lib.boolean import *
 from cvs2svn_lib.set_support import *
 from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.log import Log
-
-
-class Artifact:
-  """An artifact that can be created, used across cvs2svn passes, then
-  cleaned up."""
-
-  def __init__(self):
-    # A set of passes that need this artifact.  This field is
-    # maintained by ArtifactManager.
-    self._passes_needed = set()
-
-  def cleanup(self):
-    """This artifact is no longer needed; clean it up."""
-
-    pass
-
-
-class TempFileArtifact(Artifact):
-  """A temporary file that can be used across cvs2svn passes."""
-
-  def __init__(self, basename):
-    Artifact.__init__(self)
-    self.filename = Ctx().get_temp_filename(basename)
-
-  def cleanup(self):
-    Log().verbose("Deleting", self.filename)
-    os.unlink(self.filename)
-
-  def __str__(self):
-    return 'Temporary file %r' % self.filename
+from cvs2svn_lib.artifact import TempFile
 
 
 class ArtifactNotActiveError(Exception):
@@ -73,14 +42,14 @@ class ArtifactManager:
 
   To use this class:
 
-  - Call artifact_manager[name] = artifact once for each known
-    artifact.
+  - Call artifact_manager.set_artifact(name, artifact) once for each
+    known artifact.
 
-  - Call artifact_manager.creates(which_pass, name) to indicate that
-    WHICH_PASS is the pass that creates the artifact named NAME.
+  - Call artifact_manager.creates(which_pass, artifact) to indicate
+    that WHICH_PASS is the pass that creates ARTIFACT.
 
-  - Call artifact_manager.uses(which_pass, name) to indicate that
-    WHICH_PASS needs to use the artifact named NAME.
+  - Call artifact_manager.uses(which_pass, artifact) to indicate that
+    WHICH_PASS needs to use ARTIFACT.
 
   There are also helper methods register_temp_file(),
   register_artifact_needed(), and register_temp_file_needed() which
@@ -119,7 +88,7 @@ class ArtifactManager:
     # A set of passes that are currently being executed.
     self._active_passes = set()
 
-  def __setitem__(self, name, artifact):
+  def set_artifact(self, name, artifact):
     """Add ARTIFACT to the list of artifacts that we manage.
 
     Store it under NAME."""
@@ -127,7 +96,7 @@ class ArtifactManager:
     assert name not in self._artifacts
     self._artifacts[name] = artifact
 
-  def __getitem__(self, name):
+  def get_artifact(self, name):
     """Return the artifact with the specified name.
 
     If the artifact does not currently exist, raise a KeyError.  If it
@@ -142,23 +111,20 @@ class ArtifactManager:
     else:
       raise ArtifactNotActiveError(name)
 
-  def creates(self, which_pass, name):
-    """Register that WHICH_PASS creates the artifact named NAME.
+  def creates(self, which_pass, artifact):
+    """Register that WHICH_PASS creates ARTIFACT.
 
-    An artifact with this name must already have been registered."""
-
-    artifact = self._artifacts[name]
+    ARTIFACT must already have been registered."""
 
     # An artifact is automatically "needed" in the pass in which it is
     # created:
-    self.uses(which_pass, name)
+    self.uses(which_pass, artifact)
 
-  def uses(self, which_pass, name):
-    """Register that WHICH_PASS uses the artifact named NAME.
+  def uses(self, which_pass, artifact):
+    """Register that WHICH_PASS uses ARTIFACT.
 
-    An artifact with this name must already have been registered."""
+    ARTIFACT must already have been registered."""
 
-    artifact = self._artifacts[name]
     artifact._passes_needed.add(which_pass)
     if which_pass in self._pass_needs:
       self._pass_needs[which_pass].add(artifact)
@@ -170,19 +136,15 @@ class ArtifactManager:
 
     Return the filename of the temporary file."""
 
-    artifact = TempFileArtifact(basename)
-    self[basename] = artifact
-    self.creates(which_pass, basename)
-    return artifact.filename
-
-  def get_artifact(self, artifact_name):
-    return self[artifact_name]
+    artifact = TempFile(basename)
+    self.set_artifact(basename, artifact)
+    self.creates(which_pass, artifact)
 
   def get_temp_file(self, basename):
     """Return the filename of the temporary file with the specified BASENAME.
 
-    If the temporary file is not an existing, registered
-    TempFileArtifact, raise a KeyError."""
+    If the temporary file is not an existing, registered TempFile,
+    raise a KeyError."""
 
     return self.get_artifact(basename).filename
 
