@@ -17,6 +17,7 @@
 """This module contains common facilities used by cvs2svn."""
 
 
+import re
 import time
 import codecs
 
@@ -50,8 +51,8 @@ class FatalException(Exception):
   """Exception thrown on a non-recoverable error.
 
   If this exception is thrown by main(), it is caught by the global
-  layer of the program, its string representation is printed, and the
-  program is ended with an exit code of 1."""
+  layer of the program, its string representation is printed (followed
+  by a newline), and the program is ended with an exit code of 1."""
 
   pass
 
@@ -66,9 +67,9 @@ class FatalError(FatalException):
   """A FatalException that prepends error_prefix to the message."""
 
   def __init__(self, msg):
-    """Use (error_prefix + ': ' + MSG + '\n') as the error message."""
+    """Use (error_prefix + ': ' + MSG) as the error message."""
 
-    FatalException.__init__(self, '%s: %s\n' % (error_prefix, msg,))
+    FatalException.__init__(self, '%s: %s' % (error_prefix, msg,))
 
 
 class CommandError(FatalError):
@@ -118,6 +119,169 @@ def path_split(path):
     return (path[:pos], path[pos+1:],)
 
 
+class IllegalSVNPathError(FatalException):
+  pass
+
+
+# Control characters (characters not allowed in Subversion filenames):
+ctrl_characters_regexp = re.compile('[\\\x00-\\\x1f\\\x7f]')
+
+
+def verify_svn_filename_legal(filename):
+  """Verify that FILENAME is a legal filename.
+
+  FILENAME is a path component of a CVS path.  Check that it won't
+  choke SVN:
+
+  - Check that it is not empty.
+
+  - Check that it is not equal to '.' or '..'.
+
+  - Check that the filename does not include any control characters.
+
+  If any of these tests fail, raise an IllegalSVNPathError."""
+
+  if filename == '':
+    raise IllegalSVNPathError("Empty filename component.")
+
+  if filename in ['.', '..']:
+    raise IllegalSVNPathError("Illegal filename component %r." % (filename,))
+
+  m = ctrl_characters_regexp.search(filename)
+  if m:
+    raise IllegalSVNPathError(
+        "Character %r in filename %r is not supported by Subversion."
+        % (m.group(), filename,)
+        )
+
+
+def verify_svn_path_legal(path):
+  """Verify that PATH is a legitimate SVN path.
+
+  If not, raise an IllegalSVNPathError."""
+
+  if path.startswith('/'):
+    raise IllegalSVNPathError("Path %r must not start with '/'." % (path,))
+  head = path
+  while head != '':
+    (head,tail) = path_split(head)
+    try:
+        verify_svn_filename_legal(tail)
+    except IllegalSVNPathError, e:
+        raise IllegalSVNPathError('Problem with path %r: %s' % (path, e,))
+
+
+def normalize_svn_path(path, allow_empty=False):
+  """Normalize an SVN path (e.g., one supplied by a user).
+
+  1. Strip leading, trailing, and duplicated '/'.
+  2. If ALLOW_EMPTY is not set, verify that PATH is not empty.
+
+  Return the normalized path.
+
+  If the path is invalid, raise an IllegalSVNPathError."""
+
+  norm_path = path_join(*path.split('/'))
+  if not allow_empty and not norm_path:
+    raise IllegalSVNPathError("Path is empty")
+  return norm_path
+
+
+class PathRepeatedException(Exception):
+  def __init__(self, path, count):
+    self.path = path
+    self.count = count
+    Exception.__init__(
+        self, 'Path %s is repeated %d times' % (self.path, self.count,)
+        )
+
+
+class PathsNestedException(Exception):
+  def __init__(self, nest, nestlings):
+    self.nest = nest
+    self.nestlings = nestlings
+    Exception.__init__(
+        self,
+        'Path %s contains the following other paths: %s'
+        % (self.nest, ', '.join(self.nestlings),)
+        )
+
+
+class PathsNotDisjointException(FatalException):
+  """An exception that collects multiple other disjointness exceptions."""
+
+  def __init__(self, problems):
+    self.problems = problems
+    Exception.__init__(
+        self,
+        'The following paths are not disjoint:\n'
+        '    %s\n'
+        % ('\n    '.join([str(problem) for problem in self.problems]),)
+        )
+
+
+def verify_paths_disjoint(*paths):
+  """Verify that all of the paths in the argument list are disjoint.
+
+  If any of the paths is nested in another one (i.e., in the sense
+  that 'a/b/c/d' is nested in 'a/b'), or any two paths are identical,
+  raise a PathsNotDisjointException containing exceptions detailing
+  the individual problems."""
+
+  def split(path):
+    if not path:
+      return []
+    else:
+      return path.split('/')
+
+  def contains(split_path1, split_path2):
+    """Return True iff SPLIT_PATH1 contains SPLIT_PATH2."""
+
+    return (
+        len(split_path1) < len(split_path2)
+        and split_path2[:len(split_path1)] == split_path1
+        )
+
+  paths = [(split(path), path) for path in paths]
+  # If all overlapping elements are equal, a shorter list is
+  # considered "less than" a longer one.  Therefore if any paths are
+  # nested, this sort will leave at least one such pair adjacent, in
+  # the order [nest,nestling].
+  paths.sort()
+
+  problems = []
+
+  # Create exceptions for any repeated paths, and delete the repeats
+  # from the paths array:
+  i = 0
+  while i < len(paths):
+    split_path, path = paths[i]
+    j = i + 1
+    while j < len(paths) and split_path == paths[j][0]:
+      j += 1
+    if j - i > 1:
+      problems.append(PathRepeatedException(path, j - i))
+      # Delete all but the first copy:
+      del paths[i + 1:j]
+    i += 1
+
+  # Create exceptions for paths nested in each other:
+  i = 0
+  while i < len(paths):
+    split_path, path = paths[i]
+    j = i + 1
+    while j < len(paths) and contains(split_path, paths[j][0]):
+      j += 1
+    if j - i > 1:
+      problems.append(PathsNestedException(
+          path, [path2 for (split_path2, path2) in paths[i + 1:j]]
+          ))
+    i += 1
+
+  if problems:
+    raise PathsNotDisjointException(problems)
+
+
 def format_date(date):
   """Return an svn-compatible date string for DATE (seconds since epoch).
 
@@ -126,11 +290,11 @@ def format_date(date):
   return time.strftime("%Y-%m-%dT%H:%M:%S.000000Z", time.gmtime(date))
 
 
-class UTF8Encoder:
-  """Callable that decodes strings into unicode then encodes them as utf8."""
+class CVSTextDecoder:
+  """Callable that decodes CVS strings into unicode."""
 
   def __init__(self, encodings, fallback_encoding=None):
-    """Create a UTF8Encoder instance.
+    """Create a CVSTextDecoder instance.
 
     ENCODINGS is a list containing the names of encodings that are
     attempted to be used as source encodings in 'strict' mode.
@@ -153,22 +317,25 @@ class UTF8Encoder:
           )
 
   def __call__(self, s):
-    """Try to decode 8-bit string S using our configured source encodings.
+    """Try to decode string S using our configured source encodings.
 
-    Return the string as unicode, encoded in an 8-bit string as utf8.
+    Return the string as a unicode string.  If S is already a unicode
+    string, do nothing.
 
     Raise UnicodeError if the string cannot be decoded using any of
     the source encodings and no fallback encoding was specified."""
 
+    if isinstance(s, unicode):
+      return s
     for (name, decoder) in self.decoders:
       try:
-        return decoder(s)[0].encode('utf8')
+        return decoder(s)[0]
       except ValueError:
         Log().verbose("Encoding '%s' failed for string %r" % (name, s))
 
     if self.fallback_decoder is not None:
       (name, decoder) = self.fallback_decoder
-      return decoder(s, 'replace')[0].encode('utf8')
+      return decoder(s, 'replace')[0]
     else:
       raise UnicodeError
 
