@@ -33,6 +33,13 @@ CO_EXECUTABLE = 'co'
 CVS_EXECUTABLE = 'cvs'
 SORT_EXECUTABLE = 'sort'
 
+# A pickled list of the projects defined for this conversion.
+PROJECTS = 'projects.pck'
+
+# A file holding the Serializer to be used for
+# CVS_REVS_SUMMARY_*_DATAFILE and CVS_SYMBOLS_SYMMARY_*_DATAFILE:
+SUMMARY_SERIALIZER = 'summary-serializer.pck'
+
 # The first file contains enough information about each CVSRevision to
 # deduce preliminary Changesets.  The second file is a sorted version
 # of the first.
@@ -112,7 +119,6 @@ SYMBOL_OPENINGS_CLOSINGS_SORTED = 'symbolic-names-s.txt'
 
 # Skeleton version of an svn filesystem.  See class
 # SVNRepositoryMirror for how these work.
-SVN_MIRROR_REVISIONS_TABLE = 'svn-revisions.dat'
 SVN_MIRROR_NODES_INDEX_TABLE = 'svn-nodes-index.dat'
 SVN_MIRROR_NODES_STORE = 'svn-nodes.pck'
 
@@ -129,14 +135,6 @@ CVS_FILES_DB = 'cvs-files.pck'
 # CVSFile.
 CVS_ITEMS_STORE = 'cvs-items.pck'
 
-# A database of filtered CVSItems.  Excluded symbols have been
-# discarded (and the dependencies of the remaining CVSItems fixed up).
-# These two files are used within an IndexedCVSItemStore; the first is
-# a map id-> offset, and the second contains the pickled CVSItems at
-# the specified offsets.
-CVS_ITEMS_FILTERED_INDEX_TABLE = 'cvs-items-filtered-index.dat'
-CVS_ITEMS_FILTERED_STORE = 'cvs-items-filtered.pck'
-
 # The same as above, but with the CVSItems ordered in groups based on
 # their initial changesets.  CVSItems will usually be accessed one
 # changeset at a time, so this ordering helps disk locality (even
@@ -179,7 +177,12 @@ PIPE_READ_SIZE = 128 * 1024
 # is assigned the same id.  Note that the (author, logmessage) pairs
 # are not necessarily all distinct; other data are taken into account
 # when constructing ids.
-METADATA_DB = 'metadata.db'
+METADATA_INDEX_TABLE = 'metadata-index.dat'
+METADATA_STORE = 'metadata.pck'
+
+# The same, after it has been cleaned up for the chosen output option:
+METADATA_CLEAN_INDEX_TABLE = 'metadata-clean-index.dat'
+METADATA_CLEAN_STORE = 'metadata-clean.pck'
 
 # The following four databases are used in conjunction with --use-internal-co.
 
