@@ -56,12 +56,20 @@ from __future__ import generators
 from cvs2svn_lib.boolean import *
 from cvs2svn_lib.set_support import *
 from cvs2svn_lib.context import Ctx
+from cvs2svn_lib.symbol import Trunk
 
 
 class CVSItem(object):
-  def __init__(self, id, cvs_file):
+  __slots__ = [
+      'id',
+      'cvs_file',
+      'revision_recorder_token',
+      ]
+
+  def __init__(self, id, cvs_file, revision_recorder_token):
     self.id = id
     self.cvs_file = cvs_file
+    self.revision_recorder_token = revision_recorder_token
 
   def __eq__(self, other):
     return self.id == other.id
@@ -120,6 +128,15 @@ class CVSItem(object):
 
     raise NotImplementedError()
 
+  def check_links(self, cvs_file_items):
+    """Check for consistency of links to other CVSItems.
+
+    Other items can be looked up in CVS_FILE_ITEMS, which is an
+    instance of CVSFileItems.  Raise an AssertionError if there is a
+    problem."""
+
+    raise NotImplementedError()
+
   def __repr__(self):
     return '%s(%s)' % (self.__class__.__name__, self,)
 
@@ -131,55 +148,98 @@ class CVSRevision(CVSItem):
   a single file.
 
   Members:
-    ID -- (string) unique ID for this revision.
-    CVS_FILE -- (CVSFile) CVSFile affected by this revision.
-    TIMESTAMP -- (int) date stamp for this revision.
-    METADATA_ID -- (int) id of author + log message record in metadata_db.
-    PREV_ID -- (int) id of the logically previous CVSRevision, either on the
-        same or the source branch (or None).
-    NEXT_ID -- (int) id of the logically next CVSRevision (or None).
-    REV -- (string) the CVS revision number, e.g., '1.3'.
-    DELTATEXT_EXISTS -- (bool) true iff this revision's deltatext is not
-        empty.
-    LOD -- (LineOfDevelopment) LOD on which this revision occurred.
-    FIRST_ON_BRANCH_ID -- (int or None) if this revision is the first on its
-        branch, the cvs_branch_id of that branch; else, None.
-    DEFAULT_BRANCH_REVISION -- (bool) true iff this is a default branch
+
+    id -- (string) unique ID for this revision.
+
+    cvs_file -- (CVSFile) CVSFile affected by this revision.
+
+    timestamp -- (int) date stamp for this revision.
+
+    metadata_id -- (int) id of metadata instance record in
+        metadata_db.
+
+    prev_id -- (int) id of the logically previous CVSRevision, either
+        on the same or the source branch (or None).
+
+    next_id -- (int) id of the logically next CVSRevision (or None).
+
+    rev -- (string) the CVS revision number, e.g., '1.3'.
+
+    deltatext_exists -- (bool) true iff this revision's deltatext is
+        not empty.
+
+    lod -- (LineOfDevelopment) LOD on which this revision occurred.
+
+    first_on_branch_id -- (int or None) if this revision is the first
+        on its branch, the cvs_branch_id of that branch; else, None.
+
+    ntdbr -- (bool) true iff this is a non-trunk default branch
         revision.
-    DEFAULT_BRANCH_PREV_ID -- (int or None) Iff this is the 1.2 revision after
-        the end of a default branch, the id of the last rev on the default
-        branch; else, None.
-    DEFAULT_BRANCH_NEXT_ID -- (int or None) Iff this is the last revision on
-        a default branch preceding a 1.2 rev, the id of the 1.2 revision;
-        else, None.
-    TAG_IDS -- (list of int) ids of all CVSTags rooted at this CVSRevision.
-    BRANCH_IDS -- (list of int) ids of all CVSBranches rooted at this
+
+    ntdbr_prev_id -- (int or None) Iff this is the 1.2 revision after
+        the end of a default branch, the id of the last rev on the
+        default branch; else, None.
+
+    ntdbr_next_id -- (int or None) Iff this is the last revision on a
+        default branch preceding a 1.2 rev, the id of the 1.2
+        revision; else, None.
+
+    tag_ids -- (list of int) ids of all CVSTags rooted at this
         CVSRevision.
-    BRANCH_COMMIT_IDS -- (list of int) ids of first CVSRevision committed on
-        each branch rooted in this revision (for branches with commits).
-    OPENED_SYMBOLS -- (None or list of (symbol_id, cvs_symbol_id) tuples)
-        information about all CVSSymbols opened by this revision.  This member
-        is set in FilterSymbolsPass; before then, it is None.
-    CLOSED_SYMBOLS -- (None or list of (symbol_id, cvs_symbol_id) tuples)
-        information about all CVSSymbols closed by this revision.  This member
-        is set in FilterSymbolsPass; before then, it is None.
-    REVISION_RECORDER_TOKEN -- (arbitrary) a token that can be set by
+
+    branch_ids -- (list of int) ids of all CVSBranches rooted at this
+        CVSRevision.
+
+    branch_commit_ids -- (list of int) ids of first CVSRevision
+        committed on each branch rooted in this revision (for branches
+        with commits).
+
+    opened_symbols -- (None or list of (symbol_id, cvs_symbol_id)
+        tuples) information about all CVSSymbols opened by this
+        revision.  This member is set in FilterSymbolsPass; before
+        then, it is None.
+
+    closed_symbols -- (None or list of (symbol_id, cvs_symbol_id)
+        tuples) information about all CVSSymbols closed by this
+        revision.  This member is set in FilterSymbolsPass; before
+        then, it is None.
+
+    revision_recorder_token -- (arbitrary) a token that can be set by
         RevisionRecorder for the later use of RevisionReader.
 
   """
 
+  __slots__ = [
+      'timestamp',
+      'metadata_id',
+      'prev_id',
+      'next_id',
+      'rev',
+      'deltatext_exists',
+      'lod',
+      'first_on_branch_id',
+      'ntdbr',
+      'ntdbr_prev_id',
+      'ntdbr_next_id',
+      'tag_ids',
+      'branch_ids',
+      'branch_commit_ids',
+      'opened_symbols',
+      'closed_symbols',
+      ]
+
   def __init__(self,
                id, cvs_file,
                timestamp, metadata_id,
                prev_id, next_id,
                rev, deltatext_exists,
-               lod, first_on_branch_id, default_branch_revision,
-               default_branch_prev_id, default_branch_next_id,
+               lod, first_on_branch_id, ntdbr,
+               ntdbr_prev_id, ntdbr_next_id,
                tag_ids, branch_ids, branch_commit_ids,
                revision_recorder_token):
     """Initialize a new CVSRevision object."""
 
-    CVSItem.__init__(self, id, cvs_file)
+    CVSItem.__init__(self, id, cvs_file, revision_recorder_token)
 
     self.timestamp = timestamp
     self.metadata_id = metadata_id
@@ -189,15 +249,14 @@ class CVSRevision(CVSItem):
     self.deltatext_exists = deltatext_exists
     self.lod = lod
     self.first_on_branch_id = first_on_branch_id
-    self.default_branch_revision = default_branch_revision
-    self.default_branch_prev_id = default_branch_prev_id
-    self.default_branch_next_id = default_branch_next_id
+    self.ntdbr = ntdbr
+    self.ntdbr_prev_id = ntdbr_prev_id
+    self.ntdbr_next_id = ntdbr_next_id
     self.tag_ids = tag_ids
     self.branch_ids = branch_ids
     self.branch_commit_ids = branch_commit_ids
     self.opened_symbols = None
     self.closed_symbols = None
-    self.revision_recorder_token = revision_recorder_token
 
   def _get_cvs_path(self):
     return self.cvs_file.cvs_path
@@ -221,8 +280,8 @@ class CVSRevision(CVSItem):
         self.deltatext_exists,
         self.lod.id,
         self.first_on_branch_id,
-        self.default_branch_revision,
-        self.default_branch_prev_id, self.default_branch_next_id,
+        self.ntdbr,
+        self.ntdbr_prev_id, self.ntdbr_next_id,
         self.tag_ids, self.branch_ids, self.branch_commit_ids,
         self.opened_symbols, self.closed_symbols,
         self.revision_recorder_token,
@@ -236,14 +295,25 @@ class CVSRevision(CVSItem):
      self.deltatext_exists,
      lod_id,
      self.first_on_branch_id,
-     self.default_branch_revision,
-     self.default_branch_prev_id, self.default_branch_next_id,
+     self.ntdbr,
+     self.ntdbr_prev_id, self.ntdbr_next_id,
      self.tag_ids, self.branch_ids, self.branch_commit_ids,
      self.opened_symbols, self.closed_symbols,
      self.revision_recorder_token) = data
     self.cvs_file = Ctx()._cvs_file_db.get_file(cvs_file_id)
     self.lod = Ctx()._symbol_db.get_symbol(lod_id)
 
+  def get_effective_prev_id(self):
+    """Return the ID of the effective predecessor of this item.
+
+    This is the ID of the item that determines whether the object
+    existed before this CVSRevision."""
+
+    if self.ntdbr_prev_id is not None:
+      return self.ntdbr_prev_id
+    else:
+      return self.prev_id
+
   def get_symbol_pred_ids(self):
     """Return the pred_ids for symbol predecessors."""
 
@@ -256,8 +326,8 @@ class CVSRevision(CVSItem):
     retval = self.get_symbol_pred_ids()
     if self.prev_id is not None:
       retval.add(self.prev_id)
-    if self.default_branch_prev_id is not None:
-      retval.add(self.default_branch_prev_id)
+    if self.ntdbr_prev_id is not None:
+      retval.add(self.ntdbr_prev_id)
     return retval
 
   def get_symbol_succ_ids(self):
@@ -272,8 +342,8 @@ class CVSRevision(CVSItem):
     retval = self.get_symbol_succ_ids()
     if self.next_id is not None:
       retval.add(self.next_id)
-    if self.default_branch_next_id is not None:
-      retval.add(self.default_branch_next_id)
+    if self.ntdbr_next_id is not None:
+      retval.add(self.ntdbr_next_id)
     for id in self.branch_commit_ids:
       retval.add(id)
     return retval
@@ -339,11 +409,11 @@ class CVSRevision(CVSItem):
       # The first CVSRevision on a branch is considered to close the
       # branch:
       yield self.first_on_branch_id
-      if self.default_branch_revision:
+      if self.ntdbr:
         # If the 1.1 revision was not deleted, the 1.1.1.1 revision is
         # considered to close it:
         yield self.prev_id
-    elif self.default_branch_prev_id is not None:
+    elif self.ntdbr_prev_id is not None:
       # This is the special case of a 1.2 revision that follows a
       # non-trunk default branch.  Either 1.1 was deleted or the first
       # default branch revision closed 1.1, so we don't have to close
@@ -357,6 +427,65 @@ class CVSRevision(CVSItem):
       # prev_id is on the same LOD and this item closes that one:
       yield self.prev_id
 
+  def check_links(self, cvs_file_items):
+    assert self.cvs_file == cvs_file_items.cvs_file
+
+    prev = cvs_file_items.get(self.prev_id)
+    next = cvs_file_items.get(self.next_id)
+    first_on_branch = cvs_file_items.get(self.first_on_branch_id)
+    ntdbr_next = cvs_file_items.get(self.ntdbr_next_id)
+    ntdbr_prev = cvs_file_items.get(self.ntdbr_prev_id)
+    effective_prev = cvs_file_items.get(self.get_effective_prev_id())
+
+    if prev is None:
+      # This is the first CVSRevision on trunk or a detached branch:
+      assert self.id in cvs_file_items.root_ids
+    elif first_on_branch is not None:
+      # This is the first CVSRevision on an existing branch:
+      assert isinstance(first_on_branch, CVSBranch)
+      assert first_on_branch.symbol == self.lod
+      assert first_on_branch.next_id == self.id
+      assert first_on_branch.source_id == prev.id
+      assert self.id in prev.branch_commit_ids
+    else:
+      # This revision follows another revision on the same LOD:
+      assert prev.next_id == self.id
+      assert prev.lod == self.lod
+
+    if next is not None:
+      assert next.prev_id == self.id
+      assert next.lod == self.lod
+
+    if ntdbr_next is not None:
+      assert self.ntdbr
+      assert ntdbr_next.ntdbr_prev_id == self.id
+
+    if ntdbr_prev is not None:
+      assert ntdbr_prev.ntdbr_next_id == self.id
+
+    for tag_id in self.tag_ids:
+      tag = cvs_file_items[tag_id]
+      assert isinstance(tag, CVSTag)
+      assert tag.source_id == self.id
+      assert tag.source_lod == self.lod
+
+    branch_commit_ids = list(self.branch_commit_ids)
+    for branch_id in self.branch_ids:
+      branch = cvs_file_items[branch_id]
+      assert isinstance(branch, CVSBranch)
+      assert branch.source_id == self.id
+      assert branch.source_lod == self.lod
+      if branch.next_id is not None:
+        assert branch.next_id in branch_commit_ids
+        branch_commit_ids.remove(branch.next_id)
+    assert not branch_commit_ids
+
+    assert self.__class__ == cvs_revision_type_map[(
+        isinstance(self, CVSRevisionModification),
+        effective_prev is not None
+            and isinstance(effective_prev, CVSRevisionModification),
+        )]
+
   def __str__(self):
     """For convenience only.  The format is subject to change at any time."""
 
@@ -366,6 +495,12 @@ class CVSRevision(CVSItem):
 class CVSRevisionModification(CVSRevision):
   """Base class for CVSRevisionAdd or CVSRevisionChange."""
 
+  __slots__ = []
+
+  # Explicitly define pickle methods to get around a Python 2.2 bug:
+  __setstate__ = CVSRevision.__setstate__
+  __getstate__ = CVSRevision.__getstate__
+
   def get_cvs_symbol_ids_opened(self):
     return self.tag_ids + self.branch_ids
 
@@ -376,18 +511,32 @@ class CVSRevisionAdd(CVSRevisionModification):
   The file might have never existed on this LOD, or it might have
   existed previously but been deleted by a CVSRevisionDelete."""
 
-  pass
+  __slots__ = []
+
+  # Explicitly define pickle methods to get around a Python 2.2 bug:
+  __setstate__ = CVSRevisionModification.__setstate__
+  __getstate__ = CVSRevisionModification.__getstate__
 
 
 class CVSRevisionChange(CVSRevisionModification):
   """A CVSRevision that modifies a file that already existed on this LOD."""
 
-  pass
+  __slots__ = []
+
+  # Explicitly define pickle methods to get around a Python 2.2 bug:
+  __setstate__ = CVSRevisionModification.__setstate__
+  __getstate__ = CVSRevisionModification.__getstate__
 
 
 class CVSRevisionAbsent(CVSRevision):
   """A CVSRevision for which the file is nonexistent on this LOD."""
 
+  __slots__ = []
+
+  # Explicitly define pickle methods to get around a Python 2.2 bug:
+  __setstate__ = CVSRevision.__setstate__
+  __getstate__ = CVSRevision.__getstate__
+
   def get_cvs_symbol_ids_opened(self):
     return []
 
@@ -395,7 +544,11 @@ class CVSRevisionAbsent(CVSRevision):
 class CVSRevisionDelete(CVSRevisionAbsent):
   """A CVSRevision that deletes a file that existed on this LOD."""
 
-  pass
+  __slots__ = []
+
+  # Explicitly define pickle methods to get around a Python 2.2 bug:
+  __setstate__ = CVSRevisionAbsent.__setstate__
+  __getstate__ = CVSRevisionAbsent.__getstate__
 
 
 class CVSRevisionNoop(CVSRevisionAbsent):
@@ -407,7 +560,11 @@ class CVSRevisionNoop(CVSRevisionAbsent):
   they might have a nontrivial log message that we don't want to throw
   away."""
 
-  pass
+  __slots__ = []
+
+  # Explicitly define pickle methods to get around a Python 2.2 bug:
+  __setstate__ = CVSRevisionAbsent.__setstate__
+  __getstate__ = CVSRevisionAbsent.__getstate__
 
 
 # A map
@@ -431,20 +588,37 @@ class CVSSymbol(CVSItem):
   This is the base class for CVSBranch and CVSTag.
 
   Members:
-    ID -- (string) unique ID for this item.
-    CVS_FILE -- (CVSFile) CVSFile affected by this item.
-    SYMBOL -- (Symbol) the symbol affected by this CVSSymbol.
-    SOURCE_LOD -- (LineOfDevelopment) the LOD that is the source for this
-        CVSSymbol.
-    SOURCE_ID -- (int) the ID of the CVSRevision or CVSBranch that is the
-        source for this item.
+
+    id -- (string) unique ID for this item.
+
+    cvs_file -- (CVSFile) CVSFile affected by this item.
+
+    symbol -- (Symbol) the symbol affected by this CVSSymbol.
+
+    source_lod -- (LineOfDevelopment) the LOD that is the source for
+        this CVSSymbol.
+
+    source_id -- (int) the ID of the CVSRevision or CVSBranch that is
+        the source for this item.
+
+    revision_recorder_token -- (arbitrary) a token that can be set by
+        RevisionRecorder for the later use of RevisionReader.
 
   """
 
-  def __init__(self, id, cvs_file, symbol, source_lod, source_id):
+  __slots__ = [
+      'symbol',
+      'source_lod',
+      'source_id',
+      ]
+
+  def __init__(
+      self, id, cvs_file, symbol, source_lod, source_id,
+      revision_recorder_token
+      ):
     """Initialize a CVSSymbol object."""
 
-    CVSItem.__init__(self, id, cvs_file)
+    CVSItem.__init__(self, id, cvs_file, revision_recorder_token)
 
     self.symbol = symbol
     self.source_lod = source_lod
@@ -462,34 +636,62 @@ class CVSBranch(CVSSymbol):
   """Represent the creation of a branch in a particular CVSFile.
 
   Members:
-    ID -- (string) unique ID for this item.
-    CVS_FILE -- (CVSFile) CVSFile affected by this item.
-    SYMBOL -- (Symbol) the symbol affected by this CVSSymbol.
-    BRANCH_NUMBER -- (string) the number of this branch (e.g., '1.3.4'), or
-        None if this is a converted CVSTag.
-    SOURCE_LOD -- (LineOfDevelopment) the LOD that is the source for this
-        CVSSymbol.
-    SOURCE_ID -- (int) id of the CVSRevision or CVSBranch from which this
-        branch sprouts.
-    NEXT_ID -- (int or None) id of first CVSRevision on this branch, if any;
-        else, None.
-    TAG_IDS -- (list of int) ids of all CVSTags rooted at this CVSBranch (can
-        be set due to parent adjustment in FilterSymbolsPass).
-    BRANCH_IDS -- (list of int) ids of all CVSBranches rooted at this
-        CVSBranch (can be set due to parent adjustment in FilterSymbolsPass).
-    OPENED_SYMBOLS -- (None or list of (symbol_id, cvs_symbol_id) tuples)
-        information about all CVSSymbols opened by this branch.  This member
-        is set in FilterSymbolsPass; before then, it is None.
+
+    id -- (string) unique ID for this item.
+
+    cvs_file -- (CVSFile) CVSFile affected by this item.
+
+    symbol -- (Symbol) the symbol affected by this CVSSymbol.
+
+    branch_number -- (string) the number of this branch (e.g.,
+        '1.3.4'), or None if this is a converted CVSTag.
+
+    source_lod -- (LineOfDevelopment) the LOD that is the source for
+        this CVSSymbol.
+
+    source_id -- (int) id of the CVSRevision or CVSBranch from which
+        this branch sprouts.
+
+    next_id -- (int or None) id of first CVSRevision on this branch,
+        if any; else, None.
+
+    tag_ids -- (list of int) ids of all CVSTags rooted at this
+        CVSBranch (can be set due to parent adjustment in
+        FilterSymbolsPass).
+
+    branch_ids -- (list of int) ids of all CVSBranches rooted at this
+        CVSBranch (can be set due to parent adjustment in
+        FilterSymbolsPass).
+
+    opened_symbols -- (None or list of (symbol_id, cvs_symbol_id)
+        tuples) information about all CVSSymbols opened by this
+        branch.  This member is set in FilterSymbolsPass; before then,
+        it is None.
+
+    revision_recorder_token -- (arbitrary) a token that can be set by
+        RevisionRecorder for the later use of RevisionReader.
 
   """
 
+  __slots__ = [
+      'branch_number',
+      'next_id',
+      'tag_ids',
+      'branch_ids',
+      'opened_symbols',
+      ]
+
   def __init__(
       self, id, cvs_file, symbol, branch_number,
-      source_lod, source_id, next_id
+      source_lod, source_id, next_id,
+      revision_recorder_token,
       ):
     """Initialize a CVSBranch."""
 
-    CVSSymbol.__init__(self, id, cvs_file, symbol, source_lod, source_id)
+    CVSSymbol.__init__(
+        self, id, cvs_file, symbol, source_lod, source_id,
+        revision_recorder_token
+        )
     self.branch_number = branch_number
     self.next_id = next_id
     self.tag_ids = []
@@ -503,6 +705,7 @@ class CVSBranch(CVSSymbol):
         self.source_lod.id, self.source_id, self.next_id,
         self.tag_ids, self.branch_ids,
         self.opened_symbols,
+        self.revision_recorder_token,
         )
 
   def __setstate__(self, data):
@@ -511,7 +714,8 @@ class CVSBranch(CVSSymbol):
         symbol_id, self.branch_number,
         source_lod_id, self.source_id, self.next_id,
         self.tag_ids, self.branch_ids,
-        self.opened_symbols
+        self.opened_symbols,
+        self.revision_recorder_token,
         ) = data
     self.cvs_file = Ctx()._cvs_file_db.get_file(cvs_file_id)
     self.symbol = Ctx()._symbol_db.get_symbol(symbol_id)
@@ -529,6 +733,35 @@ class CVSBranch(CVSSymbol):
   def get_cvs_symbol_ids_opened(self):
     return self.tag_ids + self.branch_ids
 
+  def check_links(self, cvs_file_items):
+    source = cvs_file_items.get(self.source_id)
+    next = cvs_file_items.get(self.next_id)
+
+    assert self.id in source.branch_ids
+    if isinstance(source, CVSRevision):
+      assert self.source_lod == source.lod
+    elif isinstance(source, CVSBranch):
+      assert self.source_lod == source.symbol
+    else:
+      assert False
+
+    if next is not None:
+      assert isinstance(next, CVSRevision)
+      assert next.lod == self.symbol
+      assert next.first_on_branch_id == self.id
+
+    for tag_id in self.tag_ids:
+      tag = cvs_file_items[tag_id]
+      assert isinstance(tag, CVSTag)
+      assert tag.source_id == self.id
+      assert tag.source_lod == self.symbol
+
+    for branch_id in self.branch_ids:
+      branch = cvs_file_items[branch_id]
+      assert isinstance(branch, CVSBranch)
+      assert branch.source_id == self.id
+      assert branch.source_lod == self.symbol
+
   def __str__(self):
     """For convenience only.  The format is subject to change at any time."""
 
@@ -539,6 +772,12 @@ class CVSBranch(CVSSymbol):
 class CVSBranchNoop(CVSBranch):
   """A CVSBranch whose source is a CVSRevisionAbsent."""
 
+  __slots__ = []
+
+  # Explicitly define pickle methods to get around a Python 2.2 bug:
+  __setstate__ = CVSBranch.__setstate__
+  __getstate__ = CVSBranch.__getstate__
+
   def get_cvs_symbol_ids_opened(self):
     return []
 
@@ -560,29 +799,49 @@ class CVSTag(CVSSymbol):
   """Represent the creation of a tag on a particular CVSFile.
 
   Members:
-    ID -- (string) unique ID for this item.
-    CVS_FILE -- (CVSFile) CVSFile affected by this item.
-    SYMBOL -- (Symbol) the symbol affected by this CVSSymbol.
-    SOURCE_LOD -- (LineOfDevelopment) the LOD that is the source for this
-        CVSSymbol.
-    SOURCE_ID -- (int) the ID of the CVSRevision or CVSBranch that is being
-        tagged.
+
+    id -- (string) unique ID for this item.
+
+    cvs_file -- (CVSFile) CVSFile affected by this item.
+
+    symbol -- (Symbol) the symbol affected by this CVSSymbol.
+
+    source_lod -- (LineOfDevelopment) the LOD that is the source for
+        this CVSSymbol.
+
+    source_id -- (int) the ID of the CVSRevision or CVSBranch that is
+        being tagged.
+
+    revision_recorder_token -- (arbitrary) a token that can be set by
+        RevisionRecorder for the later use of RevisionReader.
 
   """
 
-  def __init__(self, id, cvs_file, symbol, source_lod, source_id):
+  __slots__ = []
+
+  def __init__(
+      self, id, cvs_file, symbol, source_lod, source_id,
+      revision_recorder_token,
+      ):
     """Initialize a CVSTag."""
 
-    CVSSymbol.__init__(self, id, cvs_file, symbol, source_lod, source_id)
+    CVSSymbol.__init__(
+        self, id, cvs_file, symbol, source_lod, source_id,
+        revision_recorder_token,
+        )
 
   def __getstate__(self):
     return (
         self.id, self.cvs_file.id, self.symbol.id,
         self.source_lod.id, self.source_id,
+        self.revision_recorder_token,
         )
 
   def __setstate__(self, data):
-    (self.id, cvs_file_id, symbol_id, source_lod_id, self.source_id) = data
+    (
+        self.id, cvs_file_id, symbol_id, source_lod_id, self.source_id,
+        self.revision_recorder_token,
+        ) = data
     self.cvs_file = Ctx()._cvs_file_db.get_file(cvs_file_id)
     self.symbol = Ctx()._symbol_db.get_symbol(symbol_id)
     self.source_lod = Ctx()._symbol_db.get_symbol(source_lod_id)
@@ -596,6 +855,17 @@ class CVSTag(CVSSymbol):
   def get_cvs_symbol_ids_opened(self):
     return []
 
+  def check_links(self, cvs_file_items):
+    source = cvs_file_items.get(self.source_id)
+
+    assert self.id in source.tag_ids
+    if isinstance(source, CVSRevision):
+      assert self.source_lod == source.lod
+    elif isinstance(source, CVSBranch):
+      assert self.source_lod == source.symbol
+    else:
+      assert False
+
   def __str__(self):
     """For convenience only.  The format is subject to change at any time."""
 
@@ -606,7 +876,11 @@ class CVSTag(CVSSymbol):
 class CVSTagNoop(CVSTag):
   """A CVSTag whose source is a CVSRevisionAbsent."""
 
-  pass
+  __slots__ = []
+
+  # Explicitly define pickle methods to get around a Python 2.2 bug:
+  __setstate__ = CVSTag.__setstate__
+  __getstate__ = CVSTag.__getstate__
 
 
 # A map
