@@ -46,13 +46,18 @@ except ImportError:
 
 # 2. These DBM modules are not good for cvs2svn.
 import anydbm
-if (anydbm._defaultmod.__name__ == 'dumbdbm'
-    or anydbm._defaultmod.__name__ == 'dbm'):
-  sys.stderr.write(
-    error_prefix
-    + ': your installation of Python does not contain a suitable\n'
-    + 'DBM module -- cvs2svn cannot continue.\n'
-    + 'See http://python.org/doc/current/lib/module-anydbm.html to solve.\n')
+if anydbm._defaultmod.__name__ in ['dumbdbm', 'dbm']:
+  Log().error(
+      '%s: cvs2svn uses the anydbm package, which depends on lower level '
+          'dbm\n'
+      'libraries.  Your system has %s, with which cvs2svn is known to have\n'
+      'problems.  To use cvs2svn, you must install a Python dbm library '
+          'other than\n'
+      'dumbdbm or dbm.  See '
+          'http://python.org/doc/current/lib/module-anydbm.html\n'
+      'for more information.\n'
+      % (error_prefix, anydbm._defaultmod.__name__,)
+      )
   sys.exit(1)
 
 # 3. If we are using the old bsddb185 module, then try prefer gdbm instead.
@@ -62,11 +67,14 @@ if hasattr(anydbm._defaultmod, 'bsddb') \
   try:
     gdbm = __import__('gdbm')
   except ImportError:
-    sys.stderr.write(warning_prefix +
-        ': The version of the bsddb module found '
-        'on your computer has been reported to malfunction on some datasets, '
-        'causing KeyError exceptions. You may wish to upgrade your Python to '
-        'version 2.3 or later.\n')
+    Log().warn(
+        '%s: The version of the bsddb module found on your computer '
+            'has been\n'
+        'reported to malfunction on some datasets, causing KeyError '
+            'exceptions.\n'
+        'You may wish to upgrade your Python to version 2.3 or later.\n'
+        % (warning_prefix,)
+        )
   else:
     anydbm._defaultmod = gdbm
 
@@ -228,7 +236,8 @@ class IndexedDatabase:
       raise RuntimeError('Invalid mode %r' % self.mode)
 
     self.index_table = RecordTable(
-        self.index_filename, self.mode, FileOffsetPacker())
+        self.index_filename, self.mode, FileOffsetPacker()
+        )
 
     if self.mode == DB_OPEN_NEW:
       assert serializer is not None
@@ -238,16 +247,31 @@ class IndexedDatabase:
       # Read the memo from the first pickle:
       self.serializer = cPickle.load(self.f)
 
+    # Seek to the end of the file, and record that position:
+    self.f.seek(0, 2)
+    self.fp = self.f.tell()
+    self.eofp = self.fp
+
   def __setitem__(self, index, item):
     """Write ITEM into the database indexed by INDEX."""
 
     # Make sure we're at the end of the file:
-    self.f.seek(0, 2)
-    self.index_table[index] = self.f.tell()
-    self.serializer.dumpf(self.f, item)
+    if self.fp != self.eofp:
+      self.f.seek(self.eofp)
+    self.index_table[index] = self.eofp
+    s = self.serializer.dumps(item)
+    self.f.write(s)
+    self.eofp += len(s)
+    self.fp = self.eofp
 
   def _fetch(self, offset):
-    self.f.seek(offset)
+    if self.fp != offset:
+      self.f.seek(offset)
+
+    # There is no easy way to tell how much data will be read, so just
+    # indicate that we don't know the current file pointer:
+    self.fp = None
+
     return self.serializer.loadf(self.f)
 
   def iterkeys(self):
@@ -267,18 +291,26 @@ class IndexedDatabase:
     except KeyError:
       return default
 
-  def get_many(self, indexes):
-    """Generate the items with the specified INDEXES in arbitrary order."""
+  def get_many(self, indexes, default=None):
+    """Yield (index,item) tuples for INDEXES, in arbitrary order.
+
+    Yield (index,default) for indexes with no defined values."""
+
+    offsets = []
+    for (index, offset) in self.index_table.get_many(indexes):
+      if offset is None:
+        yield (index, default)
+      else:
+        offsets.append((offset, index))
 
-    offsets = list(self.index_table.get_many(indexes))
     # Sort the offsets to reduce disk seeking:
     offsets.sort()
-    for offset in offsets:
-      yield self._fetch(offset)
+    for (offset,index) in offsets:
+      yield (index, self._fetch(offset))
 
   def __delitem__(self, index):
-    self.index_table[index]
-    self.index_table[index] = 0
+    # We don't actually free the data in self.f.
+    del self.index_table[index]
 
   def close(self):
     self.index_table.close()
