@@ -21,27 +21,38 @@ import os
 import md5
 
 from cvs2svn_lib.boolean import *
+from cvs2svn_lib.set_support import *
 from cvs2svn_lib import config
 from cvs2svn_lib.common import CommandError
 from cvs2svn_lib.common import FatalError
 from cvs2svn_lib.common import OP_ADD
 from cvs2svn_lib.common import OP_CHANGE
+from cvs2svn_lib.common import path_split
 from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.svn_repository_mirror import SVNRepositoryMirrorDelegate
+from cvs2svn_lib.apple_single_filter import get_maybe_apple_single_stream
 
 
 class DumpfileDelegate(SVNRepositoryMirrorDelegate):
   """Create a Subversion dumpfile."""
 
-  def __init__(self, dumpfile_path):
+  def __init__(self, revision_reader, dumpfile_path):
     """Return a new DumpfileDelegate instance, attached to a dumpfile
-    DUMPFILE_PATH, using Ctx().filename_utf8_encoder()."""
+    DUMPFILE_PATH, using Ctx().cvs_filename_decoder()."""
 
+    self._revision_reader = revision_reader
     self.dumpfile_path = dumpfile_path
 
     self.dumpfile = open(self.dumpfile_path, 'wb')
     self._write_dumpfile_header(self.dumpfile)
 
+    # A set of the basic project infrastructure project directories
+    # that have been created so far, as SVN paths.  (The root
+    # directory is considered to be present at initialization.)  This
+    # includes all of the LOD paths, and all of their parent
+    # directories etc.
+    self._basic_directories = set([''])
+
   def _write_dumpfile_header(self, dumpfile):
     # Initialize the dumpfile with the standard headers.
     #
@@ -58,9 +69,7 @@ class DumpfileDelegate(SVNRepositoryMirrorDelegate):
     # different encodings).
     for i in range(len(pieces)):
       try:
-        # Log messages can be converted with the 'replace' strategy,
-        # but we can't afford any lossiness here.
-        pieces[i] = Ctx().filename_utf8_encoder(pieces[i])
+        pieces[i] = Ctx().cvs_filename_decoder(pieces[i]).encode('utf8')
       except UnicodeError:
         raise FatalError(
             "Unable to convert a path '%s' to internal encoding.\n"
@@ -122,26 +131,73 @@ class DumpfileDelegate(SVNRepositoryMirrorDelegate):
     total_len = len(all_prop_strings)
 
     # Print the revision header and revprops
-    self.dumpfile.write('Revision-number: %d\n'
-                        'Prop-content-length: %d\n'
-                        'Content-length: %d\n'
-                        '\n'
-                        % (self.revision, total_len, total_len))
-
-    self.dumpfile.write(all_prop_strings)
-    self.dumpfile.write('\n')
+    self.dumpfile.write(
+        'Revision-number: %d\n'
+        'Prop-content-length: %d\n'
+        'Content-length: %d\n'
+        '\n'
+        '%s'
+        '\n'
+        % (self.revision, total_len, total_len, all_prop_strings)
+        )
 
   def end_commit(self):
     pass
 
-  def mkdir(self, path):
+  def _make_any_dir(self, path):
     """Emit the creation of directory PATH."""
 
-    self.dumpfile.write("Node-path: %s\n"
-                        "Node-kind: dir\n"
-                        "Node-action: add\n"
-                        "\n"
-                        "\n" % self._utf8_path(path))
+    self.dumpfile.write(
+        "Node-path: %s\n"
+        "Node-kind: dir\n"
+        "Node-action: add\n"
+        "\n"
+        "\n"
+        % self._utf8_path(path)
+        )
+
+  def _register_basic_directory(self, path, create):
+    """Register the creation of PATH if it is not already there.
+
+    Create any parent directories that do not already exist.  If
+    CREATE is set, also create PATH if it doesn't already exist.  This
+    method should only be used for the LOD paths and the directories
+    containing them, not for directories within an LOD path."""
+
+    if path not in self._basic_directories:
+      # Make sure that the parent directory is present:
+      self._register_basic_directory(path_split(path)[0], True)
+      if create:
+        self._make_any_dir(path)
+      self._basic_directories.add(path)
+
+  def initialize_project(self, project):
+    """Create the TTB directories for the project.
+
+    Be sure not to create parent directories that already exist (e.g.,
+    because two directories share part of their paths either within or
+    across projects)."""
+
+    # Find the path of the Trunk symbol for this project (which might
+    # differ from project.trunk_path because of SymbolStrategyRules):
+    trunk = Ctx()._symbol_db.get_symbol(project.trunk_id)
+
+    # For a trunk-only conversion, the trunk path might be '', in
+    # which case it already exists and we mustn't create it:
+    if trunk.base_path:
+      self._register_basic_directory(trunk.base_path, True)
+
+    if not Ctx().trunk_only:
+      self._register_basic_directory(project.branches_path, True)
+      self._register_basic_directory(project.tags_path, True)
+
+  def initialize_lod(self, lod):
+    lod_path = lod.get_path()
+    if lod_path:
+      self._register_basic_directory(lod_path, True)
+
+  def mkdir(self, lod, cvs_directory):
+    self._make_any_dir(lod.get_path(cvs_directory.cvs_path))
 
   def _add_or_change_path(self, s_item, op):
     """Emit the addition or change corresponding to S_ITEM.
@@ -208,21 +264,27 @@ class DumpfileDelegate(SVNRepositoryMirrorDelegate):
     # If the file has keywords, we must prevent CVS/RCS from expanding
     # the keywords because they must be unexpanded in the repository,
     # or Subversion will get confused.
-    stream = Ctx().revision_reader.get_content_stream(
-        cvs_rev, suppress_keyword_substitution=s_item.has_keywords())
+    stream = self._revision_reader.get_content_stream(
+        cvs_rev, suppress_keyword_substitution=s_item.has_keywords()
+        )
+
+    if Ctx().decode_apple_single:
+      # Insert a filter to decode any files that are in AppleSingle
+      # format:
+      stream = get_maybe_apple_single_stream(stream)
 
     # Insert a filter to convert all EOLs to LFs if neccessary
-    if s_item.needs_eol_filter():
-      data_reader = LF_EOL_Filter(stream)
-    else:
-      data_reader = stream
+
+    eol_style = s_item.svn_props.get('svn:eol-style', None)
+    if eol_style:
+      stream = LF_EOL_Filter(stream, eol_style)
 
     buf = None
 
     # treat .cvsignore as a directory property
     dir_path, basename = os.path.split(cvs_rev.get_svn_path())
     if basename == ".cvsignore":
-      buf = data_reader.read()
+      buf = stream.read()
       ignore_vals = generate_ignores(buf)
       ignore_contents = '\n'.join(ignore_vals)
       if ignore_contents:
@@ -233,30 +295,36 @@ class DumpfileDelegate(SVNRepositoryMirrorDelegate):
       ignore_len = len(ignore_contents)
 
       # write headers, then props
-      self.dumpfile.write('Node-path: %s\n'
-                          'Node-kind: dir\n'
-                          'Node-action: change\n'
-                          'Prop-content-length: %d\n'
-                          'Content-length: %d\n'
-                          '\n'
-                          '%s'
-                          % (self._utf8_path(dir_path), ignore_len,
-                             ignore_len, ignore_contents))
+      self.dumpfile.write(
+          'Node-path: %s\n'
+          'Node-kind: dir\n'
+          'Node-action: change\n'
+          'Prop-content-length: %d\n'
+          'Content-length: %d\n'
+          '\n'
+          '%s'
+          % (self._utf8_path(dir_path),
+             ignore_len, ignore_len, ignore_contents)
+          )
 
-    self.dumpfile.write('Node-path: %s\n'
-                        'Node-kind: file\n'
-                        'Node-action: %s\n'
-                        '%s'  # no property header if no props
-                        'Text-content-length: '
-                        % (self._utf8_path(cvs_rev.get_svn_path()),
-                           action, props_header))
+    self.dumpfile.write(
+        'Node-path: %s\n'
+        'Node-kind: file\n'
+        'Node-action: %s\n'
+        '%s'  # no property header if no props
+        % (self._utf8_path(cvs_rev.get_svn_path()), action, props_header)
+        )
 
     pos = self.dumpfile.tell()
 
-    self.dumpfile.write('0000000000000000\n'
-                        'Text-content-md5: 00000000000000000000000000000000\n'
-                        'Content-length: 0000000000000000\n'
-                        '\n')
+    content_header_fmt = (
+        'Text-content-length: %16d\n'
+        'Text-content-md5: %32s\n'
+        'Content-length: %16d\n'
+        '\n'
+        )
+
+    self.dumpfile.write(content_header_fmt % (0, '', 0,))
 
     if prop_contents:
       self.dumpfile.write(prop_contents)
@@ -265,28 +333,24 @@ class DumpfileDelegate(SVNRepositoryMirrorDelegate):
     checksum = md5.new()
     length = 0
     if buf is None:
-      buf = data_reader.read(config.PIPE_READ_SIZE)
+      buf = stream.read(config.PIPE_READ_SIZE)
     while buf != '':
       checksum.update(buf)
       length += len(buf)
       self.dumpfile.write(buf)
-      buf = data_reader.read(config.PIPE_READ_SIZE)
+      buf = stream.read(config.PIPE_READ_SIZE)
 
     stream.close()
 
-    # Go back to patch up the length and checksum headers:
+    # Go back to overwrite the length and checksum headers with the
+    # correct values.  The content length is the length of property
+    # data, text data, and any metadata around/inside around them:
     self.dumpfile.seek(pos, 0)
-    # We left 16 zeros for the text length; replace them with the real
-    # length, padded on the left with spaces:
-    self.dumpfile.write('%16d' % length)
-    # 16... + 1 newline + len('Text-content-md5: ') == 35
-    self.dumpfile.seek(pos + 35, 0)
-    self.dumpfile.write(checksum.hexdigest())
-    # 35... + 32 bytes of checksum + 1 newline + len('Content-length: ') == 84
-    self.dumpfile.seek(pos + 84, 0)
-    # The content length is the length of property data, text data,
-    # and any metadata around/inside around them.
-    self.dumpfile.write('%16d' % (length + len(prop_contents)))
+    self.dumpfile.write(
+        content_header_fmt
+        % (length, checksum.hexdigest(), length + len(prop_contents),)
+        )
+
     # Jump back to the end of the stream
     self.dumpfile.seek(0, 2)
 
@@ -305,30 +369,57 @@ class DumpfileDelegate(SVNRepositoryMirrorDelegate):
 
     self._add_or_change_path(s_item, OP_CHANGE)
 
-  def skip_path(self, cvs_rev):
-    """Ensure that the unneeded revisions are accounted for as well."""
-    Ctx().revision_reader.skip_content(cvs_rev)
+  def delete_lod(self, lod):
+    """Emit the deletion of LOD."""
 
-  def delete_path(self, path):
-    """Emit the deletion of PATH."""
+    self.dumpfile.write(
+        'Node-path: %s\n'
+        'Node-action: delete\n'
+        '\n'
+        % (self._utf8_path(lod.get_path()),)
+        )
+    self._basic_directories.remove(lod.get_path())
 
-    self.dumpfile.write('Node-path: %s\n'
-                        'Node-action: delete\n'
-                        '\n' % self._utf8_path(path))
+  def delete_path(self, lod, cvs_path):
+    self.dumpfile.write(
+        'Node-path: %s\n'
+        'Node-action: delete\n'
+        '\n'
+        % (self._utf8_path(lod.get_path(cvs_path.cvs_path)),)
+        )
+
+  def copy_lod(self, src_lod, dest_lod, src_revnum):
+    """Emit the copying of SRC_LOD at SRC_REV to DEST_LOD."""
+
+    # Register the main LOD directory, and create parent directories
+    # as needed:
+    self._register_basic_directory(dest_lod.get_path(), False)
+
+    # We don't need to include "Node-kind:" for copies; the loader
+    # ignores it anyway and just uses the source kind instead.
+    self.dumpfile.write(
+        'Node-path: %s\n'
+        'Node-action: add\n'
+        'Node-copyfrom-rev: %d\n'
+        'Node-copyfrom-path: /%s\n'
+        '\n'
+        % (self._utf8_path(dest_lod.get_path()),
+           src_revnum, self._utf8_path(src_lod.get_path()))
+        )
 
   def copy_path(self, src_path, dest_path, src_revnum):
     """Emit the copying of SRC_PATH at SRC_REV to DEST_PATH."""
 
     # We don't need to include "Node-kind:" for copies; the loader
     # ignores it anyway and just uses the source kind instead.
-    self.dumpfile.write('Node-path: %s\n'
-                        'Node-action: add\n'
-                        'Node-copyfrom-rev: %d\n'
-                        'Node-copyfrom-path: /%s\n'
-                        '\n'
-                        % (self._utf8_path(dest_path),
-                           src_revnum,
-                           self._utf8_path(src_path)))
+    self.dumpfile.write(
+        'Node-path: %s\n'
+        'Node-action: add\n'
+        'Node-copyfrom-rev: %d\n'
+        'Node-copyfrom-path: /%s\n'
+        '\n'
+        % (self._utf8_path(dest_path), src_revnum, self._utf8_path(src_path))
+        )
 
   def finish(self):
     """Perform any cleanup necessary after all revisions have been
@@ -351,10 +442,18 @@ def generate_ignores(raw_ignore_val):
 
 class LF_EOL_Filter:
   """Filter a stream and convert all end-of-line markers (CRLF, CR or LF)
-  into LFs only."""
+  into the appropriate canonical eol style."""
 
-  def __init__(self, stream):
+  eol_style_replacements = {
+      'LF' : '\n',
+      'CR' : '\r',
+      'CRLF' : '\r\n',
+      'native' : '\n',
+      }
+
+  def __init__(self, stream, eol_style):
     self.stream = stream
+    self.replacement = self.eol_style_replacements[eol_style]
     self.carry_cr = False
     self.eof = False
 
@@ -370,7 +469,13 @@ class LF_EOL_Filter:
         buf = buf[:-1]
       buf = buf.replace('\r\n', '\n')
       buf = buf.replace('\r', '\n')
+      if self.replacement != '\n':
+        buf = buf.replace('\n', self.replacement)
       if buf or self.eof:
         return buf
 
+  def close(self):
+    self.stream.close()
+    self.stream = None
+
 
