@@ -28,7 +28,6 @@ from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.artifact_manager import artifact_manager
 from cvs2svn_lib.symbol import Branch
 from cvs2svn_lib.svn_revision_range import SVNRevisionRange
-from cvs2svn_lib.symbol_filling_guide import get_source_set
 
 
 # Constants used in SYMBOL_OPENINGS_CLOSINGS
@@ -152,6 +151,11 @@ class SymbolingsReader:
     self.offsets = cPickle.load(offsets_db)
     offsets_db.close()
 
+  def close(self):
+    self.symbolings.close()
+    del self.symbolings
+    del self.offsets
+
   def _generate_lines(self, symbol):
     """Generate the lines for SYMBOL.
 
@@ -176,73 +180,62 @@ class SymbolingsReader:
 
         yield (revnum, type, cvs_symbol_id)
 
-  def _get_range_map(self, symbol):
-    """Return the ranges of all CVSSymbols related to SYMBOL.
+  def get_range_map(self, svn_symbol_commit):
+    """Return the ranges of all CVSSymbols in SVN_SYMBOL_COMMIT.
 
-    Return a map {cvs_symbol_id : SVNRevisionRange}."""
+    Return a map { CVSSymbol : SVNRevisionRange }."""
+
+    # A map { cvs_symbol_id : CVSSymbol }:
+    cvs_symbol_map = {}
+    for cvs_symbol in svn_symbol_commit.get_cvs_items():
+      cvs_symbol_map[cvs_symbol.id] = cvs_symbol
 
     range_map = {}
 
-    for (revnum, type, cvs_symbol_id) in self._generate_lines(symbol):
-      range = range_map.get(cvs_symbol_id)
+    for (revnum, type, cvs_symbol_id) \
+            in self._generate_lines(svn_symbol_commit.symbol):
+      cvs_symbol = cvs_symbol_map.get(cvs_symbol_id)
+      if cvs_symbol is None:
+        # This CVSSymbol is not part of SVN_SYMBOL_COMMIT.
+        continue
+      range = range_map.get(cvs_symbol)
       if type == OPENING:
         if range is not None:
           raise InternalError(
-              'Multiple openings logged for %r'
-              % (Ctx()._cvs_items_db[cvs_symbol_id],)
+              'Multiple openings logged for %r' % (cvs_symbol,)
               )
-        range_map[cvs_symbol_id] = SVNRevisionRange(revnum)
+        range_map[cvs_symbol] = SVNRevisionRange(
+            cvs_symbol.source_lod, revnum
+            )
       else:
         if range is None:
           raise InternalError(
-              'Closing precedes opening for %r'
-              % (Ctx()._cvs_items_db[cvs_symbol_id],)
+              'Closing precedes opening for %r' % (cvs_symbol,)
               )
         if range.closing_revnum is not None:
           raise InternalError(
-              'Multiple closings logged for %r'
-              % (Ctx()._cvs_items_db[cvs_symbol_id],)
+              'Multiple closings logged for %r' % (cvs_symbol,)
               )
         range.add_closing(revnum)
 
-    return range_map
-
-  def get_source_set(self, svn_symbol_commit, svn_revnum):
-    """Return the list of possible sources for SVN_SYMBOL_COMMIT.
-
-    SVN_SYMBOL_COMMIT is an SVNSymbolCommit instance and SVN_REVNUM is
-    an SVN revision number.  The symbol sources will contain all
-    openings and closings for CVSSymbols that occur in
-    SVN_SYMBOL_COMMIT.  SVN_REVNUM is only used for an internal
-    consistency check."""
-
-    symbol = svn_symbol_commit.symbol
-
-    range_map = self._get_range_map(svn_symbol_commit.symbol)
-
-    # A map {svn_path : SVNRevisionRange}:
-    openings_closings_map = {}
-
-    for cvs_symbol in svn_symbol_commit.get_cvs_items():
-      svn_path = cvs_symbol.source_lod.get_path(
-          cvs_symbol.cvs_file.cvs_path
-          )
+    # Make sure that all CVSSymbols are accounted for, and adjust the
+    # closings to be not later than svn_symbol_commit.revnum.
+    for cvs_symbol in cvs_symbol_map.itervalues():
       try:
-        range = range_map[cvs_symbol.id]
+        range = range_map[cvs_symbol]
       except KeyError:
         raise InternalError('No opening for %s' % (cvs_symbol,))
 
-      if range.opening_revnum >= svn_revnum:
+      if range.opening_revnum >= svn_symbol_commit.revnum:
         raise InternalError(
             'Opening in r%d not ready for %s in r%d'
-            % (range.opening_revnum, cvs_symbol, svn_revnum,)
+            % (range.opening_revnum, cvs_symbol, svn_symbol_commit.revnum,)
             )
 
-      if range.closing_revnum > svn_revnum:
+      if range.closing_revnum is not None \
+             and range.closing_revnum > svn_symbol_commit.revnum:
         range.closing_revnum = None
 
-      openings_closings_map[svn_path] = range
-
-    return get_source_set(svn_symbol_commit.symbol, openings_closings_map)
+    return range_map
 
 
