@@ -77,9 +77,11 @@ class Pass(object):
 
     artifact_manager.register_temp_file_needed(basename, self)
 
-  def run(self, stats_keeper):
+  def run(self, run_options, stats_keeper):
     """Carry out this step of the conversion.
-    STATS_KEEPER is a StatsKeeper instance."""
+
+    RUN_OPTIONS is an instance of RunOptions.  STATS_KEEPER is an
+    instance of StatsKeeper."""
 
     raise NotImplementedError
 
@@ -128,7 +130,7 @@ class PassManager:
           return i + 1
       raise InvalidPassError('Unknown pass name (%r).' % (pass_name,))
 
-  def run(self, start_pass, end_pass):
+  def run(self, run_options):
     """Run the specified passes, one after another.
 
     START_PASS is the number of the first pass that should be run.
@@ -139,8 +141,8 @@ class PassManager:
     # to execute, using the Python index range convention (i.e., first
     # pass executed and first pass *after* the ones that should be
     # executed).
-    index_start = start_pass - 1
-    index_end = end_pass
+    index_start = run_options.start_pass - 1
+    index_end = run_options.end_pass
 
     artifact_manager.register_temp_file(config.STATISTICS_FILE, self)
 
@@ -163,15 +165,22 @@ class PassManager:
     for the_pass in self.passes[0:index_start]:
       artifact_manager.pass_skipped(the_pass)
 
+    # Clear the pass timings for passes that will have to be redone:
+    for i in range(index_end, len(self.passes)):
+      stats_keeper.clear_duration_for_pass(i)
+
     start_time = time.time()
     for i in range(index_start, index_end):
       the_pass = self.passes[i]
       Log().quiet('----- pass %d (%s) -----' % (i + 1, the_pass.name,))
       artifact_manager.pass_started(the_pass)
-      the_pass.run(stats_keeper)
+      the_pass.run(run_options, stats_keeper)
       end_time = time.time()
       stats_keeper.log_duration_for_pass(
-          end_time - start_time, i + 1, the_pass.name)
+          end_time - start_time, i + 1, the_pass.name
+          )
+      stats_keeper.archive()
+      Log().normal(stats_keeper.single_pass_timing(i + 1))
       start_time = end_time
       Ctx().clean()
       # Allow the artifact manager to clean up artifacts that are no
