@@ -23,6 +23,7 @@ import types
 
 from cvs2svn_lib.boolean import *
 from cvs2svn_lib.common import FatalError
+from cvs2svn_lib.common import CommandError
 
 
 # ============================================================================
@@ -167,3 +168,25 @@ def check_command_runs(cmd, cmdname):
     raise CommandFailedException(msg)
 
 
+class PipeStream(object):
+  """A file-like object from which revision contents can be read."""
+
+  def __init__(self, pipe_command):
+    self.pipe_command = ' '.join(pipe_command)
+    self.pipe = SimplePopen(pipe_command, True)
+    self.pipe.stdin.close()
+
+  def read(self, size=None):
+    if size is None:
+      return self.pipe.stdout.read()
+    else:
+      return self.pipe.stdout.read(size)
+
+  def close(self):
+    self.pipe.stdout.close()
+    error_output = self.pipe.stderr.read()
+    exit_status = self.pipe.wait()
+    if exit_status:
+      raise CommandError(self.pipe_command, exit_status, error_output)
+
+
