@@ -19,6 +19,7 @@
 
 import sys
 import os
+import re
 import fnmatch
 import ConfigParser
 
@@ -98,17 +99,17 @@ class MimeMapper(SVNPropertySetter):
       type = extensions.pop(0)
       for ext in extensions:
         if ext in self.mappings and self.mappings[ext] != type:
-          sys.stderr.write("%s: ambiguous MIME mapping for *.%s (%s or %s)\n"
-                           % (warning_prefix, ext, self.mappings[ext], type))
+          Log().error(
+              "%s: ambiguous MIME mapping for *.%s (%s or %s)\n"
+              % (warning_prefix, ext, self.mappings[ext], type)
+              )
         self.mappings[ext] = type
 
   def set_properties(self, s_item):
     if self.propname in s_item.svn_props:
       return
 
-    basename, extension = os.path.splitext(
-        os.path.basename(s_item.cvs_rev.cvs_path)
-        )
+    basename, extension = os.path.splitext(s_item.cvs_rev.cvs_file.basename)
 
     # Extension includes the dot, so strip it (will leave extension
     # empty if filename ends with a dot, which is ok):
@@ -129,15 +130,18 @@ class AutoPropsPropertySetter(SVNPropertySetter):
   """Set arbitrary svn properties based on an auto-props configuration.
 
   This class supports case-sensitive or case-insensitive pattern
-  matching.  The 'correct' behavior is not quite clear, because
-  subversion itself does an inconsistent job of handling case in
-  auto-props patterns; see
-  http://subversion.tigris.org/issues/show_bug.cgi?id=2036.
+  matching.  The command-line default is case-insensitive behavior,
+  consistent with Subversion (see
+  http://subversion.tigris.org/issues/show_bug.cgi?id=2036).
 
   If a property specified in auto-props has already been set to a
   different value, print a warning and leave the old property value
   unchanged."""
 
+  property_unset_re = re.compile('^\!(?P<name>[^\=]+)$')
+  property_set_re = re.compile('^(?P<name>[^\!\=][^\=]*)\=(?P<value>.*)$')
+  property_novalue_re = re.compile('^(?P<name>[^\!\=][^\=]*)$')
+
   class Pattern:
     """Describes the properties to be set for files matching a pattern."""
 
@@ -179,16 +183,25 @@ class AutoPropsPropertySetter(SVNPropertySetter):
     props = value.split(';')
     propdict = {}
     for prop in props:
-      s = prop.split('=', 1)
-      if len(s) == 1:
-        propdict[s[0]] = None
-      else:
-        propdict[s[0]] = s[1]
-    self.patterns.append(
-        self.Pattern(self.transform_case(pattern), propdict))
+      m = self.property_unset_re.match(prop)
+      if m:
+        propdict[m.group('name')] = None
+        continue
 
-  def get_propdict(self, path):
-    basename = self.transform_case(os.path.basename(path))
+      m = self.property_set_re.match(prop)
+      if m:
+        propdict[m.group('name')] = m.group('value')
+        continue
+
+      m = self.property_novalue_re.match(prop)
+      if m:
+        propdict[m.group('name')] = ''
+        continue
+
+    self.patterns.append(self.Pattern(self.transform_case(pattern), propdict))
+
+  def get_propdict(self, cvs_file):
+    basename = self.transform_case(cvs_file.basename)
     propdict = {}
     for pattern in self.patterns:
       if pattern.match(basename):
@@ -197,14 +210,14 @@ class AutoPropsPropertySetter(SVNPropertySetter):
             if propdict[key] != value:
               Log().warn(
                   "Contradictory values set for property '%s' for file %s."
-                  % (key, path,))
+                  % (key, cvs_file,))
           else:
             propdict[key] = value
 
     return propdict
 
   def set_properties(self, s_item):
-    propdict = self.get_propdict(s_item.cvs_rev.cvs_path)
+    propdict = self.get_propdict(s_item.cvs_rev.cvs_file)
     for (k,v) in propdict.items():
       if k in s_item.svn_props:
         if s_item.svn_props[k] != v:
