@@ -35,16 +35,20 @@ from cvs2svn_lib import config
 from cvs2svn_lib.common import warning_prefix
 from cvs2svn_lib.common import error_prefix
 from cvs2svn_lib.common import FatalError
-from cvs2svn_lib.common import UTF8Encoder
+from cvs2svn_lib.common import CVSTextDecoder
 from cvs2svn_lib.log import Log
 from cvs2svn_lib.context import Ctx
-from cvs2svn_lib.output_option import DumpfileOutputOption
-from cvs2svn_lib.output_option import NewRepositoryOutputOption
-from cvs2svn_lib.output_option import ExistingRepositoryOutputOption
+from cvs2svn_lib.svn_output_option import DumpfileOutputOption
+from cvs2svn_lib.svn_output_option import NewRepositoryOutputOption
+from cvs2svn_lib.svn_output_option import ExistingRepositoryOutputOption
 from cvs2svn_lib.project import Project
 from cvs2svn_lib.pass_manager import InvalidPassError
-from cvs2svn_lib.revision_reader import RCSRevisionReader
-from cvs2svn_lib.revision_reader import CVSRevisionReader
+from cvs2svn_lib.revision_manager import NullRevisionRecorder
+from cvs2svn_lib.revision_manager import NullRevisionExcluder
+from cvs2svn_lib.rcs_revision_manager import RCSRevisionReader
+from cvs2svn_lib.cvs_revision_manager import CVSRevisionReader
+from cvs2svn_lib.checkout_internal import InternalRevisionRecorder
+from cvs2svn_lib.checkout_internal import InternalRevisionExcluder
 from cvs2svn_lib.checkout_internal import InternalRevisionReader
 from cvs2svn_lib.symbol_strategy import AllBranchRule
 from cvs2svn_lib.symbol_strategy import AllTagRule
@@ -53,9 +57,13 @@ from cvs2svn_lib.symbol_strategy import ExcludeRegexpStrategyRule
 from cvs2svn_lib.symbol_strategy import ForceBranchRegexpStrategyRule
 from cvs2svn_lib.symbol_strategy import ForceTagRegexpStrategyRule
 from cvs2svn_lib.symbol_strategy import HeuristicStrategyRule
-from cvs2svn_lib.symbol_strategy import RuleBasedSymbolStrategy
 from cvs2svn_lib.symbol_strategy import UnambiguousUsageRule
+from cvs2svn_lib.symbol_strategy import HeuristicPreferredParentRule
+from cvs2svn_lib.symbol_strategy import SymbolHintsFileRule
+from cvs2svn_lib.symbol_strategy import DefaultBasePathRule
+from cvs2svn_lib.symbol_transform import ReplaceSubstringsSymbolTransform
 from cvs2svn_lib.symbol_transform import RegexpSymbolTransform
+from cvs2svn_lib.symbol_transform import NormalizePathsSymbolTransform
 from cvs2svn_lib.property_setters import AutoPropsPropertySetter
 from cvs2svn_lib.property_setters import CVSBinaryFileDefaultMimeTypeSetter
 from cvs2svn_lib.property_setters import CVSBinaryFileEOLStyleSetter
@@ -110,12 +118,13 @@ history.
       --symbol-transform=P:S transform symbol names from P to S, where P and S
                              use Python regexp and reference syntax
                              respectively.  P must match the whole symbol name
+      --symbol-hints=PATH    read symbol conversion hints from PATH
       --force-branch=REGEXP  force symbols matching REGEXP to be branches
       --force-tag=REGEXP     force symbols matching REGEXP to be tags
       --exclude=REGEXP       exclude branches and tags matching REGEXP
       --symbol-default=OPT   specify how ambiguous symbols are converted.
-                             OPT is "branch", "tag", "heuristic", or
-                             "strict" (default)
+                             OPT is "heuristic" (default), "strict", "branch",
+                             or "tag"
       --no-cross-branch-commits   Prevent the creation of cross-branch commits
       --retain-conflicting-attic-files   if a file appears both in and out of
                              the CVS Attic, then leave the attic version in a
@@ -136,11 +145,11 @@ history.
 
  Extraction options:
 
-      --use-rcs              use RCS to extract revision contents (default)
+      --use-rcs              use RCS to extract revision contents
       --use-cvs              use CVS to extract revision contents
                              (only use this if having problems with RCS)
       --use-internal-co      use internal code to extract revision contents
-                             (very fast but disk space intensive)
+                             (very fast but disk space intensive) (default)
 
  Environment options:
 
@@ -164,6 +173,8 @@ history.
       --help-passes          list the available passes and their numbers
   -v, --verbose              verbose (may be specified twice for debug output)
   -q, --quiet                quiet (may be specified twice for very quiet)
+      --write-symbol-info=PATH write information and statistics about CVS
+                             symbols to PATH.
       --skip-cleanup         prevent the deletion of intermediate files
       --profile              profile with 'hotshot' (into file cvs2svn.hotshot)
 """
@@ -185,6 +196,11 @@ class RunOptions:
     self.profiling = False
     self.progname = progname
 
+    self.projects = []
+
+    # A list of one list of SymbolStrategyRules for each project:
+    self.project_symbol_strategy_rules = []
+
     try:
       self.opts, self.args = my_getopt(cmd_args, 'hvqs:p:', [
           "options=",
@@ -198,6 +214,7 @@ class RunOptions:
           "no-prune",
           "encoding=", "fallback-encoding=",
           "symbol-transform=",
+          "symbol-hints=",
           "force-branch=", "force-tag=", "exclude=", "symbol-default=",
           "no-cross-branch-commits",
           "retain-conflicting-attic-files",
@@ -217,6 +234,7 @@ class RunOptions:
 
           "version", "help", "help-passes",
           "verbose", "quiet",
+          "write-symbol-info=",
           "skip-cleanup",
           "profile",
 
@@ -225,7 +243,7 @@ class RunOptions:
           "dump-only", "create", "no-default-eol", "auto-props-ignore-case",
           ])
     except getopt.GetoptError, e:
-      sys.stderr.write(error_prefix + ': ' + str(e) + '\n\n')
+      Log().error('%s: %s\n\n' % (error_prefix, e))
       self.usage()
       sys.exit(1)
 
@@ -266,6 +284,41 @@ class RunOptions:
     # Check for problems with the options:
     self.check_options()
 
+  def add_project(
+        self,
+        project_cvs_repos_path,
+        trunk_path=None, branches_path=None, tags_path=None,
+        symbol_transforms=None,
+        symbol_strategy_rules=[],
+        ):
+    """Add a project to be converted.
+
+    Most arguments are passed straight through to the Project
+    constructor.  SYMBOL_STRATEGY_RULES is an iterable of
+    SymbolStrategyRules that will be applied to symbols in this
+    project."""
+
+    id = len(self.projects)
+    project = Project(
+        id,
+        project_cvs_repos_path,
+        trunk_path=trunk_path,
+        branches_path=branches_path, tags_path=tags_path,
+        symbol_transforms=symbol_transforms,
+        )
+
+    self.projects.append(project)
+    self.project_symbol_strategy_rules.append(list(symbol_strategy_rules))
+
+  def clear_projects(self):
+    """Clear the list of projects to be converted.
+
+    This method is for the convenience of options files, which may
+    want to import one another."""
+
+    del self.projects[:]
+    del self.project_symbol_strategy_rules[:]
+
   def process_help_options(self):
     """Process any help-type options."""
 
@@ -322,7 +375,7 @@ class RunOptions:
     use_rcs = False
     use_cvs = False
     use_internal_co = False
-    symbol_strategy_default = 'strict'
+    symbol_strategy_default = 'heuristic'
     mime_types_file = None
     auto_props_file = None
     auto_props_ignore_case = True
@@ -339,8 +392,7 @@ class RunOptions:
     force_branch = False
     force_tag = False
     symbol_transforms = []
-
-    ctx.symbol_strategy = RuleBasedSymbolStrategy()
+    symbol_strategy_rules = []
 
     for opt, value in self.opts:
       if opt  in ['-s', '--svnrepos']:
@@ -369,14 +421,16 @@ class RunOptions:
         encodings.insert(-1, value)
       elif opt == '--fallback-encoding':
         fallback_encoding = value
+      elif opt == '--symbol-hints':
+        symbol_strategy_rules.append(SymbolHintsFileRule(value))
       elif opt == '--force-branch':
-        ctx.symbol_strategy.add_rule(ForceBranchRegexpStrategyRule(value))
+        symbol_strategy_rules.append(ForceBranchRegexpStrategyRule(value))
         force_branch = True
       elif opt == '--force-tag':
-        ctx.symbol_strategy.add_rule(ForceTagRegexpStrategyRule(value))
+        symbol_strategy_rules.append(ForceTagRegexpStrategyRule(value))
         force_tag = True
       elif opt == '--exclude':
-        ctx.symbol_strategy.add_rule(ExcludeRegexpStrategyRule(value))
+        symbol_strategy_rules.append(ExcludeRegexpStrategyRule(value))
       elif opt == '--symbol-default':
         if value not in ['branch', 'tag', 'heuristic', 'strict']:
           raise FatalError(
@@ -429,6 +483,8 @@ class RunOptions:
         keywords_off = True
       elif opt == '--tmpdir':
         ctx.tmpdir = value
+      elif opt == '--write-symbol-info':
+        ctx.symbol_info_filename = value
       elif opt == '--skip-cleanup':
         ctx.skip_cleanup = True
       elif opt == '--svnadmin':
@@ -441,13 +497,17 @@ class RunOptions:
         ctx.sort_executable = value
       elif opt == '--dump-only':
         dump_only = True
-        sys.stderr.write(warning_prefix +
+        Log().error(
+            warning_prefix +
             ': The --dump-only option is deprecated (it is implied\n'
-            'by --dumpfile).\n')
+            'by --dumpfile).\n'
+            )
       elif opt == '--create':
-        sys.stderr.write(warning_prefix +
+        Log().error(
+            warning_prefix +
             ': The behaviour produced by the --create option is now the '
-            'default,\nand passing the option is deprecated.\n')
+            'default,\nand passing the option is deprecated.\n'
+            )
 
     # Consistency check for options and arguments.
     if len(self.args) == 0:
@@ -455,8 +515,7 @@ class RunOptions:
       sys.exit(1)
 
     if len(self.args) > 1:
-      sys.stderr.write(error_prefix +
-                       ": must pass only one CVS repository.\n")
+      Log().error(error_prefix + ": must pass only one CVS repository.\n")
       self.usage()
       sys.exit(1)
 
@@ -517,39 +576,54 @@ class RunOptions:
       ctx.output_option = DumpfileOutputOption(dumpfile)
 
     if use_rcs:
+      ctx.revision_recorder = NullRevisionRecorder()
+      ctx.revision_excluder = NullRevisionExcluder()
       ctx.revision_reader = RCSRevisionReader(co_executable)
     elif use_cvs:
+      ctx.revision_recorder = NullRevisionRecorder()
+      ctx.revision_excluder = NullRevisionExcluder()
       ctx.revision_reader = CVSRevisionReader(cvs_executable)
     else:
       # --use-internal-co is the default:
+      ctx.revision_recorder = InternalRevisionRecorder(compress=True)
+      ctx.revision_excluder = InternalRevisionExcluder()
       ctx.revision_reader = InternalRevisionReader(compress=True)
 
-    # Create the default project (using ctx.trunk, ctx.branches, and
-    # ctx.tags):
-    ctx.add_project(Project(
-        cvsroot, trunk_base, branches_base, tags_base,
-        symbol_transforms=symbol_transforms))
-
     try:
-      ctx.utf8_encoder = UTF8Encoder(encodings, fallback_encoding)
+      ctx.cvs_author_decoder = CVSTextDecoder(encodings, fallback_encoding)
+      ctx.cvs_log_decoder = CVSTextDecoder(encodings, fallback_encoding)
       # Don't use fallback_encoding for filenames:
-      ctx.filename_utf8_encoder = UTF8Encoder(encodings)
+      ctx.cvs_filename_decoder = CVSTextDecoder(encodings)
     except LookupError, e:
       raise FatalError(str(e))
 
-    ctx.symbol_strategy.add_rule(UnambiguousUsageRule())
+    # Add the standard symbol name cleanup rules:
+    symbol_transforms.extend([
+        ReplaceSubstringsSymbolTransform('\\','/'),
+        # Remove leading, trailing, and repeated slashes:
+        NormalizePathsSymbolTransform(),
+        ])
+
+    symbol_strategy_rules.append(UnambiguousUsageRule())
     if symbol_strategy_default == 'strict':
       pass
     elif symbol_strategy_default == 'branch':
-      ctx.symbol_strategy.add_rule(AllBranchRule())
+      symbol_strategy_rules.append(AllBranchRule())
     elif symbol_strategy_default == 'tag':
-      ctx.symbol_strategy.add_rule(AllTagRule())
+      symbol_strategy_rules.append(AllTagRule())
     elif symbol_strategy_default == 'heuristic':
-      ctx.symbol_strategy.add_rule(BranchIfCommitsRule())
-      ctx.symbol_strategy.add_rule(HeuristicStrategyRule())
+      symbol_strategy_rules.append(BranchIfCommitsRule())
+      symbol_strategy_rules.append(HeuristicStrategyRule())
     else:
       assert False
 
+    # Now add a rule that sets the SVN path for each LOD:
+    symbol_strategy_rules.append(DefaultBasePathRule())
+
+    # Now add a rule whose job it is to pick the preferred parents of
+    # branches and tags:
+    symbol_strategy_rules.append(HeuristicPreferredParentRule())
+
     if auto_props_file:
       ctx.svn_property_setters.append(AutoPropsPropertySetter(
           auto_props_file, auto_props_ignore_case))
@@ -574,6 +648,17 @@ class RunOptions:
 
     ctx.svn_property_setters.append(ExecutablePropertySetter())
 
+    # Create the default project (using ctx.trunk, ctx.branches, and
+    # ctx.tags):
+    self.add_project(
+        cvsroot,
+        trunk_path=trunk_base,
+        branches_path=branches_base,
+        tags_path=tags_base,
+        symbol_transforms=symbol_transforms,
+        symbol_strategy_rules=symbol_strategy_rules,
+        )
+
   def check_options(self):
     """Check the the run options are OK.
 
@@ -592,7 +677,7 @@ class RunOptions:
     if ctx.output_option is not None:
       ctx.output_option.check()
 
-    if not ctx.projects:
+    if not self.projects:
       raise FatalError('No project specified.')
 
   def get_options(self, *names):
@@ -623,18 +708,22 @@ class RunOptions:
 
     if self.opts or self.args:
       if self.opts:
-        sys.stderr.write(
+        Log().error(
             '%s: The following options cannot be used in combination with '
             'the --options\n'
             'option:\n'
             '    %s\n'
-            % (error_prefix,
-               '\n    '.join([opt for (opt,value) in self.opts])))
+            % (
+                error_prefix,
+                '\n    '.join([opt for (opt,value) in self.opts])
+                )
+            )
       if self.args:
-        sys.stderr.write(
+        Log().error(
             '%s: No cvs-repos-path arguments are allowed with the --options '
             'option.\n'
-            % (error_prefix,))
+            % (error_prefix,)
+            )
       sys.exit(1)
 
   def process_options_file(self, options_filename):
@@ -650,7 +739,7 @@ class RunOptions:
     execfile(options_filename, g, l)
 
   def usage(self):
-    sys.stdout.write(usage_message_template % {
+    Log().write(usage_message_template % {
         'progname' : self.progname,
         'trunk_base' : config.DEFAULT_TRUNK_BASE,
         'branches_base' : config.DEFAULT_BRANCHES_BASE,
