@@ -23,18 +23,40 @@ from cvs2svn_lib.boolean import *
 from cvs2svn_lib.set_support import *
 from cvs2svn_lib import config
 from cvs2svn_lib.common import error_prefix
-from cvs2svn_lib.common import InternalError
+from cvs2svn_lib.common import FatalException
 from cvs2svn_lib.log import Log
 from cvs2svn_lib.artifact_manager import artifact_manager
+from cvs2svn_lib.symbol import LineOfDevelopment
 from cvs2svn_lib.symbol import Trunk
 from cvs2svn_lib.symbol import Symbol
+from cvs2svn_lib.symbol import IncludedSymbol
+from cvs2svn_lib.symbol import Branch
 from cvs2svn_lib.symbol import Tag
 from cvs2svn_lib.symbol import ExcludedSymbol
-from cvs2svn_lib.symbol import TypedSymbol
 from cvs2svn_lib.cvs_item import CVSBranch
 from cvs2svn_lib.cvs_item import CVSTag
 
 
+class SymbolPlanError(FatalException):
+  pass
+
+
+class SymbolPlanException(SymbolPlanError):
+  def __init__(self, stats, symbol, msg):
+    self.stats = stats
+    self.symbol = symbol
+    SymbolPlanError.__init__(
+        self,
+        'Cannot convert the following symbol to %s: %s\n    %s'
+        % (symbol, msg, self.stats,)
+        )
+
+
+class IndeterminateSymbolException(SymbolPlanException):
+  def __init__(self, stats, symbol):
+    SymbolPlanException.__init__(self, stats, symbol, 'Indeterminate type')
+
+
 class _Stats:
   """A summary of information about a symbol (tag or branch).
 
@@ -51,6 +73,10 @@ class _Stats:
     branch_commit_count -- the number of files in which there were
         commits on this lod
 
+    pure_ntdb_count -- the number of files in which this branch was
+        purely a non-trunk default branch (consisting only of
+        non-trunk default branch revisions).
+
     branch_blockers -- a set of Symbol instances for any symbols that
         sprout from a branch with this name.
 
@@ -64,6 +90,7 @@ class _Stats:
     self.branch_create_count = 0
     self.branch_commit_count = 0
     self.branch_blockers = set()
+    self.pure_ntdb_count = 0
     self.possible_parents = { }
 
   def register_tag_creation(self):
@@ -89,6 +116,11 @@ class _Stats:
 
     self.branch_blockers.add(blocker)
 
+  def register_pure_ntdb(self):
+    """Register that this branch is a pure import branch in one file."""
+
+    self.pure_ntdb_count += 1
+
   def register_possible_parent(self, lod):
     """Register that LOD was a possible parent for SELF.lod in a file."""
 
@@ -148,37 +180,56 @@ class _Stats:
         and not self.possible_parents
         )
 
-  def get_preferred_parents(self):
-    """Return the LinesOfDevelopment preferred as parents for this lod.
+  def check_valid(self, symbol):
+    """Check whether SYMBOL is a valid conversion of SELF.lod.
 
-    Return the tuple (BEST_SYMBOLS, BEST_COUNT), where BEST_SYMBOLS is
-    the set of LinesOfDevelopment that appeared most often as possible
-    parents, and BEST_COUNT is the number of times those symbols
-    appeared.  BEST_SYMBOLS might contain multiple symbols if multiple
-    LinesOfDevelopment have the same count."""
+    It is planned to convert SELF.lod as SYMBOL.  Verify that SYMBOL
+    is a TypedSymbol and that the information that it contains is
+    consistent with that stored in SELF.lod.  (This routine does not
+    do higher-level tests of whether the chosen conversion is actually
+    sensible.)  If there are any problems, raise a
+    SymbolPlanException."""
 
-    best_count = -1
-    best_symbols = set()
-    for (symbol, count) in self.possible_parents.items():
-      if count > best_count:
-        best_count = count
-        best_symbols.clear()
-        best_symbols.add(symbol)
-      elif count == best_count:
-        best_symbols.add(symbol)
+    if not isinstance(symbol, (Trunk, Branch, Tag, ExcludedSymbol)):
+      raise IndeterminateSymbolException(self, symbol)
 
-    return (best_symbols, best_count)
+    if symbol.id != self.lod.id:
+      raise SymbolPlanException(self, symbol, 'IDs must match')
+
+    if symbol.project != self.lod.project:
+      raise SymbolPlanException(self, symbol, 'Projects must match')
+
+    if isinstance(symbol, IncludedSymbol) and symbol.name != self.lod.name:
+      raise SymbolPlanException(self, symbol, 'Names must match')
+
+  def check_preferred_parent_allowed(self, symbol):
+    """Check that SYMBOL's preferred_parent_id is an allowed parent.
+
+    SYMBOL is the planned conversion of SELF.lod.  Verify that its
+    preferred_parent_id is a possible parent of SELF.lod.  If not,
+    raise a SymbolPlanException describing the problem."""
+
+    if isinstance(symbol, IncludedSymbol) \
+           and symbol.preferred_parent_id is not None:
+      for pp in self.possible_parents.keys():
+        if pp.id == symbol.preferred_parent_id:
+          return
+      else:
+        raise SymbolPlanException(
+            self, symbol,
+            'The selected parent is not among the symbol\'s '
+            'possible parents.'
+            )
 
   def __str__(self):
     return (
-        '\'%s\' is a tag in %d files, a branch in '
-        '%d files and has commits in %d files'
-        % (self.lod, self.tag_create_count,
-           self.branch_create_count, self.branch_commit_count))
+        '\'%s\' is a tag in %d files, a branch in %d files, '
+        'a pure import in %d files, and has commits in %d files'
+        % (self.lod, self.tag_create_count, self.branch_create_count,
+           self.pure_ntdb_count, self.branch_commit_count))
 
   def __repr__(self):
-    retval = ['%s; %d possible parents:\n'
-              % (self, len(self.possible_parents))]
+    retval = ['%s\n  possible parents:\n' % (self,)]
     parent_counts = self.possible_parents.items()
     parent_counts.sort(lambda a,b: - cmp(a[1], b[1]))
     for (symbol, count) in parent_counts:
@@ -186,6 +237,12 @@ class _Stats:
         retval.append('    trunk : %d\n' % count)
       else:
         retval.append('    \'%s\' : %d\n' % (symbol.name, count))
+    if self.branch_blockers:
+      blockers = list(self.branch_blockers)
+      blockers.sort()
+      retval.append('  blockers:\n')
+      for blocker in blockers:
+        retval.append('    \'%s\'\n' % (blocker,))
     return ''.join(retval)
 
 
@@ -223,7 +280,7 @@ class SymbolStatisticsCollector:
       return stats
 
   def register(self, cvs_file_items):
-    """Register the possible parents for each symbol in CVS_FILE_ITEMS."""
+    """Register the statistics for each symbol in CVS_FILE_ITEMS."""
 
     for lod_items in cvs_file_items.iter_lods():
       if lod_items.lod is not None:
@@ -244,6 +301,9 @@ class SymbolStatisticsCollector:
           branch_stats.register_branch_possible_parents(
               lod_items.cvs_branch, cvs_file_items)
 
+        if lod_items.is_pure_ntdb():
+          branch_stats.register_pure_ntdb()
+
       for cvs_tag in lod_items.cvs_tags:
         tag_stats = self[cvs_tag.symbol]
 
@@ -300,14 +360,22 @@ class SymbolStatistics:
     # development:
     self._stats = { }
 
+    # A map { LineOfDevelopment.id -> _Stats } for all lines of
+    # development:
+    self._stats_by_id = { }
+
     stats_list = cPickle.load(open(filename, 'rb'))
 
     for stats in stats_list:
       self._stats[stats.lod] = stats
+      self._stats_by_id[stats.lod.id] = stats
 
   def __len__(self):
     return len(self._stats)
 
+  def __getitem__(self, lod_id):
+    return self._stats_by_id[lod_id]
+
   def get_stats(self, lod):
     """Return the _Stats object for LineOfDevelopment instance LOD.
 
@@ -318,133 +386,121 @@ class SymbolStatistics:
   def __iter__(self):
     return self._stats.itervalues()
 
-  def _find_blocked_excludes(self, symbols):
-    """Find all excluded symbols that are blocked by non-excluded symbols.
+  def _check_blocked_excludes(self, symbol_map):
+    """Check for any excluded LODs that are blocked by non-excluded symbols.
 
-    Non-excluded symbols are by definition the symbols contained in
-    SYMBOLS, which is a map { name : Symbol } not including Trunk
-    entries.  Return a map { name : blocker_names } containing any
-    problems found, where blocker_names is a set containing the names
-    of blocking symbols."""
+    If any are found, describe the problem to Log().error() and raise
+    a FatalException."""
 
-    blocked_branches = {}
-    for stats in self:
-      if isinstance(stats.lod, Trunk):
-        # Trunk is never excluded
-        pass
-      elif stats.lod.name not in symbols:
-        blockers = [ blocker.name for blocker in stats.branch_blockers
-                     if blocker.name in symbols ]
-        if blockers:
-          blocked_branches[stats.lod.name] = set(blockers)
-    return blocked_branches
+    # A list of (lod,[blocker,...]) tuples for excludes that are
+    # blocked by the specified non-excluded blockers:
+    problems = []
 
-  def _check_blocked_excludes(self, symbols):
-    """Check whether any excluded branches are blocked.
+    for lod in symbol_map.itervalues():
+      if isinstance(lod, ExcludedSymbol):
+        # Symbol is excluded; make sure that its blockers are also
+        # excluded:
+        lod_blockers = []
+        for blocker in self.get_stats(lod).branch_blockers:
+          if isinstance(symbol_map.get(blocker, None), IncludedSymbol):
+            lod_blockers.append(blocker)
+        if lod_blockers:
+          problems.append((lod, lod_blockers))
 
-    SYMBOLS is a map { name : Symbol } not including Trunk entries.  A
-    branch can be blocked because it has another, non-excluded symbol
-    that depends on it.  If any blocked excludes are found in SYMBOLS,
-    output error messages describing the situation.  Return True if
-    any errors were found."""
+    if problems:
+      s = []
+      for (lod, lod_blockers) in problems:
+        s.append(
+            '%s: %s cannot be excluded because the following symbols '
+                'depend on it:\n'
+            % (error_prefix, lod,)
+            )
+        for blocker in lod_blockers:
+          s.append('    %s\n' % (blocker,))
+      s.append('\n')
+      Log().error(''.join(s))
 
-    Log().quiet("Checking for blocked exclusions...")
+      raise FatalException()
 
-    blocked_excludes = self._find_blocked_excludes(symbols)
-    if not blocked_excludes:
-      return False
-
-    for branch, branch_blockers in blocked_excludes.items():
-      sys.stderr.write(error_prefix + ": The branch '%s' cannot be "
-                       "excluded because the following symbols depend "
-                       "on it:\n" % (branch))
-      for blocker in branch_blockers:
-        sys.stderr.write("    '%s'\n" % (blocker))
-    sys.stderr.write("\n")
-    return True
-
-  def _check_invalid_tags(self, symbols):
+  def _check_invalid_tags(self, symbol_map):
     """Check for commits on any symbols that are to be converted as tags.
 
-    SYMBOLS is a map { name : Symbol } not including Trunk entries.
-    If there is a commit on a symbol, then it cannot be converted as a
-    tag.  If any tags with commits are found, output error messages
-    describing the problems.  Return True iff any errors are found."""
+    SYMBOL_MAP is a map {AbstractSymbol : (Trunk|TypedSymbol)}
+    indicating how each AbstractSymbol is to be converted.  If there
+    is a commit on a symbol, then it cannot be converted as a tag.  If
+    any tags with commits are found, output error messages describing
+    the problems then raise a FatalException."""
 
     Log().quiet("Checking for forced tags with commits...")
 
     invalid_tags = [ ]
-    for symbol in symbols.values():
+    for symbol in symbol_map.itervalues():
       if isinstance(symbol, Tag):
         stats = self.get_stats(symbol)
         if stats.branch_commit_count > 0:
-          invalid_tags.append(stats.lod.name)
+          invalid_tags.append(symbol)
 
     if not invalid_tags:
       # No problems found:
-      return False
+      return
 
-    sys.stderr.write(error_prefix + ": The following branches cannot be "
-                     "forced to be tags because they have commits:\n")
+    s = []
+    s.append(
+        '%s: The following branches cannot be forced to be tags '
+        'because they have commits:\n'
+        % (error_prefix,)
+        )
     for tag in invalid_tags:
-      sys.stderr.write("    '%s'\n" % (tag))
-    sys.stderr.write("\n")
+      s.append('    %s\n' % (tag.name))
+    s.append('\n')
+    Log().error(''.join(s))
 
-    return True
+    raise FatalException()
 
-  def check_consistency(self, lods):
+  def check_consistency(self, symbol_map):
     """Check the plan for how to convert symbols for consistency.
 
-    LODS is an iterable of Trunk and TypedSymbol objects indicating
-    how each line of development is to be converted.  Return True iff
-    any problems were detected."""
+    SYMBOL_MAP is a map {AbstractSymbol : (Trunk|TypedSymbol)}
+    indicating how each AbstractSymbol is to be converted.  If any
+    problems are detected, describe the problem to Log().error() and
+    raise a FatalException."""
 
-    # Keep track of which symbols have not yet been processed:
-    unprocessed_lods = set(self._stats.keys())
+    # We want to do all of the consistency checks even if one of them
+    # fails, so that the user gets as much feedback as possible.  Set
+    # this variable to True if any errors are found.
+    error_found = False
 
-    # Create a map { symbol_name : Symbol } including only
-    # non-excluded symbols:
-    symbols_by_name = {}
-    for lod in lods:
-      try:
-        unprocessed_lods.remove(lod)
-      except KeyError:
-        if lod in self._stats:
-          raise InternalError(
-              'Symbol %s appeared twice in the symbol conversion table'
-              % (lod,))
-        else:
-          raise InternalError('Symbol %s is unknown' % (lod,))
+    # Check that the planned preferred parents are OK for all
+    # IncludedSymbols:
+    for lod in symbol_map.itervalues():
+      if isinstance(lod, IncludedSymbol):
+        stats = self.get_stats(lod)
+        try:
+          stats.check_preferred_parent_allowed(lod)
+        except SymbolPlanException, e:
+          Log().error('%s\n' % (e,))
+          error_found = True
 
-      if isinstance(lod, Trunk):
-        # Trunk is not processed any further.
-        pass
-      elif isinstance(lod, ExcludedSymbol):
-        # Symbol excluded; don't process it any further.
-        pass
-      elif isinstance(lod, TypedSymbol):
-        # This is an included symbol.  Include it in the symbol check.
-        symbols_by_name[lod.name] = lod
-      else:
-        raise InternalError('Symbol %s is of unexpected type' % (lod,))
+    try:
+      self._check_blocked_excludes(symbol_map)
+    except FatalException:
+      error_found = True
 
-    # Make sure that all symbols were processed:
-    if unprocessed_lods:
-        raise InternalError(
-            'The following symbols did not appear in the symbol conversion '
-            'table: %s'
-            % (', '.join([str(s) for s in unprocessed_lods]),))
+    try:
+      self._check_invalid_tags(symbol_map)
+    except FatalException:
+      error_found = True
 
-    # It is important that we not short-circuit here:
-    return (
-        self._check_blocked_excludes(symbols_by_name)
-        | self._check_invalid_tags(symbols_by_name)
-        )
+    if error_found:
+      raise FatalException(
+          'Please fix the above errors and restart CollateSymbolsPass'
+          )
 
   def exclude_symbol(self, symbol):
     """SYMBOL has been excluded; remove it from our statistics."""
 
     del self._stats[symbol]
+    del self._stats_by_id[symbol.id]
 
     # Remove references to this symbol from other statistics objects:
     for stats in self._stats.itervalues():
@@ -452,29 +508,4 @@ class SymbolStatistics:
       if symbol in stats.possible_parents:
         del stats.possible_parents[symbol]
 
-  def get_preferred_parents(self):
-    """Return the LinesOfDevelopment preferred as parents for each symbol.
-
-    Return a map {Symbol : LineOfDevelopment} giving the LOD that
-    appears most often as a possible parent for each symbol.  Do not
-    include entries for Trunk objects.  If a symbol has no possible
-    parents because it never exists as a CVSBranch or a CVSTag, then
-    the associated value is None."""
-
-    retval = {}
-    for stats in self._stats.itervalues():
-      if isinstance(stats.lod, Trunk):
-        # Trunk entries don't have any parents.
-        pass
-      else:
-        (parents, count) = stats.get_preferred_parents()
-        if not parents:
-          retval[stats.lod] = None
-        else:
-          parents = list(parents)
-          parents.sort()
-          retval[stats.lod] = parents[0]
-
-    return retval
-
 
