@@ -18,15 +18,16 @@
 import os, shutil, re, sys, errno
 import difflib, pprint
 import xml.parsers.expat
+from xml.dom.minidom import parseString
 
-import main, verify, tree, wc  # general svntest routines in this module.
+import main, verify, tree, wc
 from svntest import Failure
 
 def no_sleep_for_timestamps():
-  os.environ['SVN_SLEEP_FOR_TIMESTAMPS'] = 'no'
+  os.environ['SVN_I_LOVE_CORRUPTED_WORKING_COPIES_SO_DISABLE_SLEEP_FOR_TIMESTAMPS'] = 'yes'
 
 def do_sleep_for_timestamps():
-  os.environ['SVN_SLEEP_FOR_TIMESTAMPS'] = 'yes'
+  os.environ['SVN_I_LOVE_CORRUPTED_WORKING_COPIES_SO_DISABLE_SLEEP_FOR_TIMESTAMPS'] = 'no'
 
 def setup_pristine_repository():
   """Create the pristine repository and 'svn import' the greek tree"""
@@ -53,11 +54,10 @@ def setup_pristine_repository():
     # import the greek tree, using l:foo/p:bar
     ### todo: svn should not be prompting for auth info when using
     ### repositories with no auth/auth requirements
-    output, errput = main.run_svn(None, 'import',
-                                  '--username', main.wc_author,
-                                  '--password', main.wc_passwd,
-                                  '-m', 'Log message for revision 1.',
-                                  main.greek_dump_dir, main.pristine_url)
+    exit_code, output, errput = main.run_svn(None, 'import', '-m',
+                                             'Log message for revision 1.',
+                                             main.greek_dump_dir,
+                                             main.pristine_url)
 
     # check for any errors from the import
     if len(errput):
@@ -86,7 +86,7 @@ def setup_pristine_repository():
     expected_output_tree = tree.build_generic_tree(output_list)
 
     try:
-      tree.compare_trees(output_tree, expected_output_tree)
+      tree.compare_trees("output", output_tree, expected_output_tree)
     except tree.SVNTreeUnequal:
       verify.display_trees("ERROR:  output of import command is unexpected.",
                            "OUTPUT TREE", expected_output_tree, output_tree)
@@ -118,18 +118,102 @@ def guarantee_greek_repository(path):
   main.chmod_tree(path, 0666, 0666)
 
 
-def run_and_verify_svnversion(message, wc_dir, repo_url,
-                              expected_stdout, expected_stderr):
-  "Run svnversion command and check its output"
-  out, err = main.run_svnversion(wc_dir, repo_url)
+def run_and_verify_svnlook(message, expected_stdout,
+                           expected_stderr, *varargs):
+  """Like run_and_verify_svnlook2, but the expected exit code is
+  assumed to be 0 if no output is expected on stderr, and 1 otherwise.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+  expected_exit = 0
+  if expected_stderr is not None and expected_stderr != []:
+    expected_exit = 1
+  return run_and_verify_svnlook2(message, expected_stdout, expected_stderr,
+                                 expected_exit, *varargs)
+
+def run_and_verify_svnlook2(message, expected_stdout, expected_stderr,
+                            expected_exit, *varargs):
+  """Run svnlook command and check its output and exit code.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+  exit_code, out, err = main.run_svnlook(*varargs)
   verify.verify_outputs("Unexpected output", out, err,
                         expected_stdout, expected_stderr)
-  return out, err
+  verify.verify_exit_code(message, exit_code, expected_exit)
+  return exit_code, out, err
 
 
+def run_and_verify_svnadmin(message, expected_stdout,
+                            expected_stderr, *varargs):
+  """Like run_and_verify_svnadmin2, but the expected exit code is
+  assumed to be 0 if no output is expected on stderr, and 1 otherwise.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+  expected_exit = 0
+  if expected_stderr is not None and expected_stderr != []:
+    expected_exit = 1
+  return run_and_verify_svnadmin2(message, expected_stdout, expected_stderr,
+                                  expected_exit, *varargs)
+
+def run_and_verify_svnadmin2(message, expected_stdout, expected_stderr,
+                             expected_exit, *varargs):
+  """Run svnadmin command and check its output and exit code.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+  exit_code, out, err = main.run_svnadmin(*varargs)
+  verify.verify_outputs("Unexpected output", out, err,
+                        expected_stdout, expected_stderr)
+  verify.verify_exit_code(message, exit_code, expected_exit)
+  return exit_code, out, err
+
+
+def run_and_verify_svnversion(message, wc_dir, repo_url,
+                              expected_stdout, expected_stderr):
+  """like run_and_verify_svnversion2, but the expected exit code is
+  assumed to be 0 if no output is expected on stderr, and 1 otherwise.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+  expected_exit = 0
+  if expected_stderr is not None and expected_stderr != []:
+    expected_exit = 1
+  return run_and_verify_svnversion2(message, wc_dir, repo_url,
+                                    expected_stdout, expected_stderr,
+                                    expected_exit)
+
+def run_and_verify_svnversion2(message, wc_dir, repo_url,
+                               expected_stdout, expected_stderr,
+                               expected_exit):
+  """Run svnversion command and check its output and exit code.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+  exit_code, out, err = main.run_svnversion(wc_dir, repo_url)
+  verify.verify_outputs("Unexpected output", out, err,
+                        expected_stdout, expected_stderr)
+  verify.verify_exit_code(message, exit_code, expected_exit)
+  return exit_code, out, err
+
 def run_and_verify_svn(message, expected_stdout, expected_stderr, *varargs):
-  """Invokes main.run_svn() with *VARARGS, return stdout and stderr as
-  lists of lines.  For both EXPECTED_STDOUT and EXPECTED_STDERR,
+  """like run_and_verify_svn2, but the expected exit code is assumed to
+  be 0 if no output is expected on stderr, and 1 otherwise.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+
+  expected_exit = 0
+  if expected_stderr is not None and expected_stderr != []:
+    expected_exit = 1
+  return run_and_verify_svn2(message, expected_stdout, expected_stderr,
+                             expected_exit, *varargs)
+
+def run_and_verify_svn2(message, expected_stdout, expected_stderr,
+                        expected_exit, *varargs):
+  """Invokes main.run_svn() with *VARARGS, returns exit code as int, stdout
+  and stderr as lists of lines.  For both EXPECTED_STDOUT and EXPECTED_STDERR,
   create an appropriate instance of verify.ExpectedOutput (if necessary):
 
      - If it is an array of strings, create a vanilla ExpectedOutput.
@@ -146,34 +230,76 @@ def run_and_verify_svn(message, expected_stdout, expected_stderr, *varargs):
   If EXPECTED_STDOUT is None, do not check stdout.
   EXPECTED_STDERR may not be None.
 
+  If output checks pass, on supported platforms (namely those with the Popen3
+  class), the expected and actual codes are compared.  On platforms lacking
+  Popen3, the actual exit code is unavailable and a value of None is returned
+  as the exit code from this and all other run_...() functions.
+
   If a comparison fails, a Failure will be raised."""
 
   if expected_stderr is None:
     raise verify.SVNIncorrectDatatype("expected_stderr must not be None")
 
   want_err = None
-  if expected_stderr is not None and expected_stderr is not []:
+  if expected_stderr is not None and expected_stderr != []:
     want_err = True
 
-  out, err = main.run_svn(want_err, *varargs)
+  exit_code, out, err = main.run_svn(want_err, *varargs)
   verify.verify_outputs(message, out, err, expected_stdout, expected_stderr)
-  return out, err
+  verify.verify_exit_code(message, exit_code, expected_exit)
+  return exit_code, out, err
+
+def run_and_verify_svn_match_any(message, expected_stdout, expected_stderr,
+                                 *varargs):
+  """Like run_and_verify_svn_match_any2, but the expected exit code is
+  assumed to be 0 if no output is expected on stderr, and 1 otherwise.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+  expected_exit = 0
+  if expected_stderr is not None and expected_stderr != []:
+    expected_exit = 1
+  return run_and_verify_svn_match_any2(message, expected_stdout,
+                                       expected_stderr, expected_exit,
+                                       *varargs)
+
+
+def run_and_verify_svn_match_any2(message, expected_stdout, expected_stderr,
+                                 expected_exit, *varargs):
+  """Like run_and_verify_svn2, except that only one stdout line must match
+  EXPECTED_STDOUT.
+
+  Exit code is not checked on platforms without Popen3 - see note in
+  run_and_verify_svn2."""
+
+  if expected_stderr is None:
+    raise verify.SVNIncorrectDatatype("expected_stderr must not be None")
+
+  want_err = None
+  if expected_stderr is not None and expected_stderr != []:
+    want_err = True
+
+  exit_code, out, err = main.run_svn(want_err, *varargs)
+  verify.verify_outputs(message, out, err, expected_stdout, expected_stderr,
+                        False)
+  verify.verify_exit_code(message, exit_code, expected_exit)
+  return exit_code, out, err
 
 
 def run_and_verify_load(repo_dir, dump_file_content):
   "Runs 'svnadmin load' and reports any errors."
   expected_stderr = []
-  output, errput = \
-          main.run_command_stdin(main.svnadmin_binary,
-    expected_stderr, 1, dump_file_content,
+  exit_code, output, errput = main.run_command_stdin(
+    main.svnadmin_binary, expected_stderr, 1, dump_file_content,
     'load', '--force-uuid', '--quiet', repo_dir)
+
   verify.verify_outputs("Unexpected stderr output", None, errput,
                         None, expected_stderr)
 
 
 def run_and_verify_dump(repo_dir):
   "Runs 'svnadmin dump' and reports any errors, returning the dump content."
-  output, errput = main.run_svnadmin('dump', repo_dir)
+  exit_code, output, errput = main.run_svnadmin('dump', repo_dir)
   verify.verify_outputs("Missing expected output(s)", output, errput,
                         verify.AnyOutput, verify.AnyOutput)
   return output
@@ -220,9 +346,10 @@ def run_and_verify_checkout(URL, wc_dir_name, output_tree, disk_tree,
 
   The subcommand output will be verified against OUTPUT_TREE,
   and the working copy itself will be verified against DISK_TREE.
-  SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
-  tree.compare_trees - see that function's doc string for more details.
-  Returns if successful and raise on failure.
+  For the latter comparison, SINGLETON_HANDLER_A and
+  SINGLETON_HANDLER_B will be passed to tree.compare_trees -- see that
+  function's doc string for more details.  Return if successful, raise
+  on failure.
 
   WC_DIR_NAME is deleted if present unless the '--force' option is passed
   in *ARGS."""
@@ -246,20 +373,18 @@ def run_and_verify_checkout(URL, wc_dir_name, output_tree, disk_tree,
   # Checkout and make a tree of the output, using l:foo/p:bar
   ### todo: svn should not be prompting for auth info when using
   ### repositories with no auth/auth requirements
-  output, errput = main.run_svn (None, 'co',
-                                 '--username', main.wc_author,
-                                 '--password', main.wc_passwd,
-                                 URL, wc_dir_name, *args)
+  exit_code, output, errput = main.run_svn(None, 'co',
+                                           URL, wc_dir_name, *args)
   actual = tree.build_tree_from_checkout (output)
 
   # Verify actual output against expected output.
-  tree.compare_trees (actual, output_tree)
+  tree.compare_trees ("output", actual, output_tree)
 
   # Create a tree by scanning the working copy
   actual = tree.build_tree_from_wc (wc_dir_name)
 
   # Verify expected disk against actual disk.
-  tree.compare_trees (actual, disk_tree,
+  tree.compare_trees ("disk", actual, disk_tree,
                       singleton_handler_a, a_baton,
                       singleton_handler_b, b_baton)
 
@@ -274,9 +399,10 @@ def run_and_verify_export(URL, export_dir_name, output_tree, disk_tree,
 
   The subcommand output will be verified against OUTPUT_TREE,
   and the exported copy itself will be verified against DISK_TREE.
-  SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
-  tree.compare_trees - see that function's doc string for more details.
-  Returns if successful and raise on failure."""
+  For the latter comparison, SINGLETON_HANDLER_A and
+  SINGLETON_HANDLER_B will be passed to tree.compare_trees -- see that
+  function's doc string for more details.  Return if successful, raise
+  on failure."""
 
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
@@ -286,14 +412,12 @@ def run_and_verify_export(URL, export_dir_name, output_tree, disk_tree,
   # Export and make a tree of the output, using l:foo/p:bar
   ### todo: svn should not be prompting for auth info when using
   ### repositories with no auth/auth requirements
-  output, errput = main.run_svn (None, 'export',
-                                 '--username', main.wc_author,
-                                 '--password', main.wc_passwd,
-                                 URL, export_dir_name, *args)
+  exit_code, output, errput = main.run_svn(None, 'export',
+                                           URL, export_dir_name, *args)
   actual = tree.build_tree_from_checkout (output)
 
   # Verify actual output against expected output.
-  tree.compare_trees (actual, output_tree)
+  tree.compare_trees ("output", actual, output_tree)
 
   # Create a tree by scanning the working copy.  Don't ignore
   # the .svn directories so that we generate an error if they
@@ -301,7 +425,7 @@ def run_and_verify_export(URL, export_dir_name, output_tree, disk_tree,
   actual = tree.build_tree_from_wc (export_dir_name, ignore_svn=False)
 
   # Verify expected disk against actual disk.
-  tree.compare_trees (actual, disk_tree,
+  tree.compare_trees ("disk", actual, disk_tree,
                       singleton_handler_a, a_baton,
                       singleton_handler_b, b_baton)
 
@@ -440,13 +564,15 @@ def run_and_verify_log_xml(message=None, expected_paths=None,
   if expected_paths != None:
     log_args.append('-v')
 
-  (stdout, stderr) = run_and_verify_svn(
+  (exit_code, stdout, stderr) = run_and_verify_svn(
     message, expected_stdout, expected_stderr,
     'log', '--xml', *log_args)
   if not parse:
     return
 
-  for (index, entry) in enumerate(LogParser().parse(stdout)):
+  entries = LogParser().parse(stdout)
+  for index in xrange(len(entries)):
+    entry = entries[index]
     if expected_revprops != None:
       entry.assert_revprops(expected_revprops[index])
     if expected_paths != None:
@@ -461,31 +587,57 @@ def verify_update(actual_output, wc_dir_name,
   """Verify update of WC_DIR_NAME.
 
   The subcommand output (found in ACTUAL_OUTPUT) will be verified
-  against OUTPUT_TREE, and the working copy itself will be verified
-  against DISK_TREE.  If optional STATUS_TREE is given, then
-  'svn status' output will be compared.  (This is a good way to check
-  that revision numbers were bumped.)  SINGLETON_HANDLER_A and
-  SINGLETON_HANDLER_B will be passed to tree.compare_trees - see that
-  function's doc string for more details.  If CHECK_PROPS is set, then
-  disk comparison will examine props.  Returns if successful, raises
-  on failure."""
+  against OUTPUT_TREE (if provided), the working copy itself will be
+  verified against DISK_TREE (if provided), and the working copy's
+  'svn status' output will be verified against STATUS_TREE (if
+  provided).  (This is a good way to check that revision numbers were
+  bumped.)
+
+  Return if successful, raise on failure.
+
+  For the comparison with DISK_TREE, pass SINGLETON_HANDLER_A and
+  SINGLETON_HANDLER_B to tree.compare_trees -- see that function's doc
+  string for more details.  If CHECK_PROPS is set, then disk
+  comparison will examine props."""
 
   # Verify actual output against expected output.
-  tree.compare_trees (actual_output, output_tree)
+  if output_tree:
+    tree.compare_trees ("output", actual_output, output_tree)
 
-  # Create a tree by scanning the working copy
-  actual_disk = tree.build_tree_from_wc (wc_dir_name, check_props)
-
-  # Verify expected disk against actual disk.
-  tree.compare_trees (actual_disk, disk_tree,
-                      singleton_handler_a, a_baton,
-                      singleton_handler_b, b_baton)
+  # Create a tree by scanning the working copy, and verify it
+  if disk_tree:
+    actual_disk = tree.build_tree_from_wc (wc_dir_name, check_props)
+    tree.compare_trees ("disk", actual_disk, disk_tree,
+                        singleton_handler_a, a_baton,
+                        singleton_handler_b, b_baton)
 
   # Verify via 'status' command too, if possible.
   if status_tree:
     run_and_verify_status(wc_dir_name, status_tree)
 
 
+def verify_disk(wc_dir_name,
+                disk_tree,
+                singleton_handler_a = None,
+                a_baton = None,
+                singleton_handler_b = None,
+                b_baton = None,
+                check_props = False):
+
+  """Verify WC_DIR_NAME against DISK_TREE.  SINGLETON_HANDLER_A,
+  A_BATON, SINGLETON_HANDLER_B, and B_BATON will be passed to
+  tree.compare_trees, which see for details.  If CHECK_PROPS is set,
+  the comparison will examin props.  Returns if successful, raises on
+  failure."""
+  if isinstance(disk_tree, wc.State):
+    disk_tree = disk_tree.old_tree()
+  verify_update (None, wc_dir_name, None, disk_tree, None,
+                 singleton_handler_a, a_baton,
+                 singleton_handler_b, b_baton,
+                 check_props)
+                 
+  
+
 def run_and_verify_update(wc_dir_name,
                           output_tree, disk_tree, status_tree,
                           error_re_string = None,
@@ -512,12 +664,14 @@ def run_and_verify_update(wc_dir_name,
   working copy itself will be verified against DISK_TREE.  If optional
   STATUS_TREE is given, then 'svn status' output will be compared.
   (This is a good way to check that revision numbers were bumped.)
-  SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
-  tree.compare_trees - see that function's doc string for more
-  details.
+
+  For the DISK_TREE verification, SINGLETON_HANDLER_A and
+  SINGLETON_HANDLER_B will be passed to tree.compare_trees -- see that
+  function's doc string for more details.
 
   If CHECK_PROPS is set, then disk comparison will examine props.
-  Returns if successful, raises on failure."""
+
+  Return if successful, raise on failure."""
 
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
@@ -528,9 +682,11 @@ def run_and_verify_update(wc_dir_name,
 
   # Update and make a tree of the output.
   if len(args):
-    output, errput = main.run_svn (error_re_string, 'up', *args)
+    exit_code, output, errput = main.run_svn(error_re_string, 'up', *args)
   else:
-    output, errput = main.run_svn (error_re_string, 'up', wc_dir_name, *args)
+    exit_code, output, errput = main.run_svn(error_re_string,
+                                             'up', wc_dir_name,
+                                             *args)
 
   if (error_re_string):
     rm = re.compile(error_re_string)
@@ -597,16 +753,17 @@ def run_and_verify_merge2(dir, rev1, rev2, url1, url2,
   working copy itself will be verified against DISK_TREE.  If optional
   STATUS_TREE is given, then 'svn status' output will be compared.
   The 'skipped' merge output will be compared to SKIP_TREE.
-  SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
-  tree.compare_trees - see that function's doc string for more
-  details.
+
+  For the DISK_TREE verification, SINGLETON_HANDLER_A and
+  SINGLETON_HANDLER_B will be passed to tree.compare_trees -- see that
+  function's doc string for more details.
 
   If CHECK_PROPS is set, then disk comparison will examine props.
 
   If DRY_RUN is set then a --dry-run merge will be carried out first and
   the output compared with that of the full merge.
 
-  Returns if successful, raises on failure."""
+  Return if successful, raise on failure."""
 
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
@@ -631,10 +788,11 @@ def run_and_verify_merge2(dir, rev1, rev2, url1, url2,
     pre_disk = tree.build_tree_from_wc(dir)
     dry_run_command = merge_command + ('--dry-run',)
     dry_run_command = dry_run_command + args
-    out_dry, err_dry = main.run_svn(error_re_string, *dry_run_command)
+    exit_code, out_dry, err_dry = main.run_svn(error_re_string,
+                                               *dry_run_command)
     post_disk = tree.build_tree_from_wc(dir)
     try:
-      tree.compare_trees(post_disk, pre_disk)
+      tree.compare_trees("disk", post_disk, pre_disk)
     except tree.SVNTreeError:
       print "============================================================="
       print "Dry-run merge altered working copy"
@@ -644,7 +802,7 @@ def run_and_verify_merge2(dir, rev1, rev2, url1, url2,
 
   # Update and make a tree of the output.
   merge_command = merge_command + args
-  out, err = main.run_svn (error_re_string, *merge_command)
+  exit_code, out, err = main.run_svn(error_re_string, *merge_command)
 
   if error_re_string:
     if not error_re_string.startswith(".*"):
@@ -677,7 +835,7 @@ def run_and_verify_merge2(dir, rev1, rev2, url1, url2,
     raise Failure
 
   myskiptree = tree.build_tree_from_skipped(out)
-  tree.compare_trees(myskiptree, skip_tree,
+  tree.compare_trees("skip", myskiptree, skip_tree,
                      extra_skip, None, missing_skip, None)
 
   actual = tree.build_tree_from_checkout(out, 0)
@@ -688,6 +846,44 @@ def run_and_verify_merge2(dir, rev1, rev2, url1, url2,
                  check_props)
 
 
+def run_and_verify_mergeinfo(error_re_string = None,
+                             expected_output = [],
+                             *args):
+  """Run 'svn mergeinfo ARGS', and compare the result against
+  EXPECTED_OUTPUT, a list of revisions expected in the output.
+  Raise an exception if an unexpected output is encountered."""
+
+  mergeinfo_command = ["mergeinfo"]
+  mergeinfo_command.extend(args)
+  exit_code, out, err = main.run_svn(error_re_string, *mergeinfo_command)
+
+  if error_re_string:
+    if not error_re_string.startswith(".*"):
+      error_re_string = ".*(" + error_re_string + ")"
+    expected_err = verify.RegexOutput(error_re_string, match_all=False)
+    verify.verify_outputs(None, None, err, None, expected_err)
+    return
+
+  out = filter(None, map(lambda x: int(x.rstrip()[1:]), out))
+  out.sort()
+  expected_output.sort()
+
+  extra_out = []
+  if out != expected_output:
+    exp_hash = dict.fromkeys(expected_output)
+    for rev in out:
+      if exp_hash.has_key(rev):
+        del(exp_hash[rev])
+      else:
+        extra_out.append(rev)
+    extra_exp = exp_hash.keys()
+    raise Exception("Unexpected 'svn mergeinfo' output:\n"
+                    "  expected but not found: %s\n"
+                    "  found but not expected: %s"
+                    % (', '.join(map(lambda x: str(x), extra_exp)),
+                       ', '.join(map(lambda x: str(x), extra_out))))
+
+
 def run_and_verify_switch(wc_dir_name,
                           wc_target,
                           switch_url,
@@ -711,10 +907,15 @@ def run_and_verify_switch(wc_dir_name,
   working copy itself will be verified against DISK_TREE.  If optional
   STATUS_TREE is given, then 'svn status' output will be
   compared.  (This is a good way to check that revision numbers were
-  bumped.)  SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
-  tree.compare_trees - see that function's doc string for more details.
-  If CHECK_PROPS is set, then disk comparison will examine props.
-  Returns if successful, raises on failure."""
+  bumped.)
+
+  For the DISK_TREE verification, SINGLETON_HANDLER_A and
+  SINGLETON_HANDLER_B will be passed to tree.compare_trees -- see that
+  function's doc string for more details.
+
+  If CHECK_PROPS is set, then disk comparison will examine props. 
+
+  Return if successful, raise on failure."""
 
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
@@ -724,10 +925,8 @@ def run_and_verify_switch(wc_dir_name,
     status_tree = status_tree.old_tree()
 
   # Update and make a tree of the output.
-  output, errput = main.run_svn (error_re_string, 'switch',
-                                 '--username', main.wc_author,
-                                 '--password', main.wc_passwd,
-                                 switch_url, wc_target, *args)
+  exit_code, output, errput = main.run_svn(error_re_string, 'switch',
+                                           switch_url, wc_target, *args)
 
   if error_re_string:
     if not error_re_string.startswith(".*"):
@@ -749,10 +948,6 @@ def run_and_verify_switch(wc_dir_name,
 
 def run_and_verify_commit(wc_dir_name, output_tree, status_tree,
                           error_re_string = None,
-                          singleton_handler_a = None,
-                          a_baton = None,
-                          singleton_handler_b = None,
-                          b_baton = None,
                           *args):
   """Commit and verify results within working copy WC_DIR_NAME,
   sending ARGS to the commit subcommand.
@@ -766,9 +961,7 @@ def run_and_verify_commit(wc_dir_name, output_tree, status_tree,
   ERROR_RE_STRING is a string, the commit must exit with error, and
   the error message must match regular expression ERROR_RE_STRING.
 
-  SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
-  tree.compare_trees - see that function's doc string for more
-  details.  Returns if successful, raises on failure."""
+  Return if successful, raise on failure."""
 
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
@@ -776,11 +969,9 @@ def run_and_verify_commit(wc_dir_name, output_tree, status_tree,
     status_tree = status_tree.old_tree()
 
   # Commit.
-  output, errput = main.run_svn(error_re_string, 'ci',
-                                '--username', main.wc_author,
-                                '--password', main.wc_passwd,
-                                '-m', 'log msg',
-                                *args)
+  exit_code, output, errput = main.run_svn(error_re_string, 'ci',
+                                           '-m', 'log msg',
+                                           *args)
 
   if error_re_string:
     if not error_re_string.startswith(".*"):
@@ -823,7 +1014,7 @@ def run_and_verify_commit(wc_dir_name, output_tree, status_tree,
 
   # Verify actual output against expected output.
   try:
-    tree.compare_trees (actual, output_tree)
+    tree.compare_trees ("output", actual, output_tree)
   except tree.SVNTreeError:
       verify.display_trees("Output of commit is unexpected",
                            "OUTPUT TREE", output_tree, actual)
@@ -850,16 +1041,14 @@ def run_and_verify_status(wc_dir_name, output_tree,
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
 
-  output, errput = main.run_svn (None, 'status', '-v', '-u', '-q',
-                                 '--username', main.wc_author,
-                                 '--password', main.wc_passwd,
-                                 wc_dir_name)
+  exit_code, output, errput = main.run_svn(None, 'status', '-v', '-u', '-q',
+                                           wc_dir_name)
 
   actual = tree.build_tree_from_status (output)
 
   # Verify actual output against expected output.
   try:
-    tree.compare_trees (actual, output_tree,
+    tree.compare_trees ("status", actual, output_tree,
                         singleton_handler_a, a_baton,
                         singleton_handler_b, b_baton)
   except tree.SVNTreeError:
@@ -883,18 +1072,93 @@ def run_and_verify_unquiet_status(wc_dir_name, output_tree,
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
 
-  output, errput = main.run_svn (None, 'status', '-v', '-u', wc_dir_name)
+  exit_code, output, errput = main.run_svn(None, 'status', '-v',
+                                           '-u', wc_dir_name)
 
   actual = tree.build_tree_from_status (output)
 
   # Verify actual output against expected output.
   if (singleton_handler_a or singleton_handler_b):
-    tree.compare_trees (actual, output_tree,
+    tree.compare_trees ("output", actual, output_tree,
                         singleton_handler_a, a_baton,
                         singleton_handler_b, b_baton)
   else:
-    tree.compare_trees (actual, output_tree)
+    tree.compare_trees ("output", actual, output_tree)
 
+def run_and_verify_diff_summarize_xml(error_re_string = [],
+                                      expected_prefix = None,
+                                      expected_paths = [],
+                                      expected_items = [],
+                                      expected_props = [],
+                                      expected_kinds = [],
+                                      *args):
+  """Run 'diff --summarize --xml' with the arguments *ARGS, which should
+  contain all arguments beyond for your 'diff --summarize --xml' omitting
+  said arguments.  EXPECTED_PREFIX will store a "common" path prefix
+  expected to be at the beginning of each summarized path.  If
+  EXPECTED_PREFIX is None, then EXPECTED_PATHS will need to be exactly
+  as 'svn diff --summarize --xml' will output.  If ERROR_RE_STRING, the
+  command must exit with error, and the error message must match regular
+  expression ERROR_RE_STRING.
+
+  Else if ERROR_RE_STRING is None, the subcommand output will be parsed
+  into an XML document and will then be verified by comparing the parsed
+  output to the contents in the EXPECTED_PATHS, EXPECTED_ITEMS,
+  EXPECTED_PROPS and EXPECTED_KINDS. Returns on success, raises
+  on failure."""
+
+  exit_code, output, errput = run_and_verify_svn(None, None, error_re_string,
+                                                 'diff', '--summarize',
+                                                 '--xml', *args)
+
+
+  # Return if errors are present since they were expected
+  if len(errput) > 0:
+    return
+
+  doc = parseString(''.join(output))
+  paths = doc.getElementsByTagName("path")
+  items = expected_items
+  kinds = expected_kinds
+
+  for path in paths:
+    modified_path = path.childNodes[0].data
+
+    if (expected_prefix is not None
+        and modified_path.find(expected_prefix) == 0):
+      modified_path = modified_path.replace(expected_prefix, '')[1:].strip()
+
+    # Workaround single-object diff
+    if len(modified_path) == 0:
+      modified_path = path.childNodes[0].data.split(os.sep)[-1]
+
+    # From here on, we use '/' as path separator.
+    if os.sep != "/":
+      modified_path = modified_path.replace(os.sep, "/")
+
+    if modified_path not in expected_paths:
+      print "ERROR: %s not expected in the changed paths." % modified_path
+      raise Failure
+
+    index = expected_paths.index(modified_path)
+    expected_item = items[index]
+    expected_kind = kinds[index]
+    expected_prop = expected_props[index]
+    actual_item = path.getAttribute('item')
+    actual_kind = path.getAttribute('kind')
+    actual_prop = path.getAttribute('props')
+  
+    if expected_item != actual_item:
+      print "ERROR: expected:", expected_item, "actual:", actual_item
+      raise Failure
+
+    if expected_kind != actual_kind:
+      print "ERROR: expected:", expected_kind, "actual:", actual_kind
+      raise Failure
+
+    if expected_prop != actual_prop:
+      print "ERROR: expected:", expected_prop, "actual:", actual_prop
+      raise Failure
 
 def run_and_verify_diff_summarize(output_tree, error_re_string = None,
                                   singleton_handler_a = None,
@@ -915,10 +1179,8 @@ def run_and_verify_diff_summarize(output_tree, error_re_string = None,
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
 
-  output, errput = main.run_svn (None, 'diff', '--summarize',
-                                 '--username', main.wc_author,
-                                 '--password', main.wc_passwd,
-                                 *args)
+  exit_code, output, errput = main.run_svn(None, 'diff', '--summarize',
+                                           *args)
 
   if error_re_string:
     if not error_re_string.startswith(".*"):
@@ -931,14 +1193,14 @@ def run_and_verify_diff_summarize(output_tree, error_re_string = None,
 
   # Verify actual output against expected output.
   try:
-    tree.compare_trees (actual, output_tree,
+    tree.compare_trees ("output", actual, output_tree,
                         singleton_handler_a, a_baton,
                         singleton_handler_b, b_baton)
   except tree.SVNTreeError:
     verify.display_trees(None, 'DIFF OUTPUT TREE', output_tree, actual)
     raise
 
-def run_and_validate_lock(path, username, password):
+def run_and_validate_lock(path, username):
   """`svn lock' the given path and validate the contents of the lock.
      Use the given username. This is important because locks are
      user specific."""
@@ -948,13 +1210,12 @@ def run_and_validate_lock(path, username, password):
   # lock the path
   run_and_verify_svn(None, ".*locked by user", [], 'lock',
                      '--username', username,
-                     '--password', password,
                      '-m', comment, path)
 
   # Run info and check that we get the lock fields.
-  output, err = run_and_verify_svn(None, None, [],
-                                   'info','-R',
-                                   path)
+  exit_code, output, err = run_and_verify_svn(None, None, [],
+                                              'info','-R',
+                                              path)
 
   ### TODO: Leverage RegexOuput([...], match_all=True) here.
   # prepare the regexs to compare against
@@ -976,16 +1237,23 @@ def run_and_validate_lock(path, username, password):
 
 
 # This allows a test to *quickly* bootstrap itself.
-def make_repo_and_wc(sbox, create_wc = True):
+def make_repo_and_wc(sbox, create_wc = True, read_only = False):
   """Create a fresh repository and checkout a wc from it.
 
-  The repo and wc directories will both be named TEST_NAME, and
-  repsectively live within the global dirs 'general_repo_dir' and
-  'general_wc_dir' (variables defined at the top of this test
-  suite.)  Returns on success, raises on failure."""
+  If read_only is False, a dedicated repository will be created, named 
+  TEST_NAME. The repository will live in the global dir 'general_repo_dir'.
+  If read_only is True the pristine repository will be used.
+
+  If create_wc is True, a dedicated working copy will be checked out from
+  the repository, named TEST_NAME. The wc directory will live in the global
+  dir 'general_wc_dir'.
+
+  Both variables 'general_repo_dir' and 'general_wc_dir' are defined at the 
+  top of this test suite.)  Returns on success, raises on failure."""
 
   # Create (or copy afresh) a new repos with a greek tree in it.
-  guarantee_greek_repository(sbox.repo_dir)
+  if not read_only:
+    guarantee_greek_repository(sbox.repo_dir)
 
   if create_wc:
     # Generate the expected output tree.
@@ -1057,17 +1325,15 @@ pre-revprop-change hook script and (if appropriate) making it executable."""
   hook_path = main.get_pre_revprop_change_hook_path (repo_dir)
   main.create_python_hook_script (hook_path, 'import sys; sys.exit(0)')
 
-def disable_revprop_changes(repo_dir, message):
+def disable_revprop_changes(repo_dir):
   """Disable revprop changes in a repository REPO_DIR by creating a
 pre-revprop-change hook script like enable_revprop_changes, except that
-the hook prints MESSAGE to stderr and exits non-zero.  MESSAGE is printed
-very simply, and should have no newlines or quotes."""
-
+the hook prints "pre-revprop-change" followed by sys.argv"""
   hook_path = main.get_pre_revprop_change_hook_path (repo_dir)
   main.create_python_hook_script (hook_path,
                                   'import sys\n'
-                                  'sys.stderr.write("%s")\n'
-                                  'sys.exit(1)\n' % (message,))
+                                  'sys.stderr.write("pre-revprop-change %s %s %s %s %s" % (sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5]))\n'
+                                  'sys.exit(1)\n')
 
 def create_failing_post_commit_hook(repo_dir):
   """Disable commits in a repository REPOS_DIR by creating a post-commit hook
@@ -1092,9 +1358,10 @@ def set_prop(expected_err, name, value, path, valp):
 def check_prop(name, path, exp_out):
   """Verify that property NAME on PATH has a value of EXP_OUT"""
   # Not using run_svn because binary_mode must be set
-  out, err = main.run_command(main.svn_binary, None, 1, 'pg', '--strict',
-                              name, path, '--config-dir',
-                              main.default_config_dir)
+  exit_code, out, err = main.run_command(main.svn_binary, None, 1, 'pg',
+                                         '--strict', name, path,
+                                         '--config-dir',
+                                         main.default_config_dir)
   if out != exp_out:
     print "svn pg --strict", name, "output does not match expected."
     print "Expected standard output: ", exp_out, "\n"
@@ -1145,14 +1412,11 @@ def inject_conflict_into_wc(sbox, state_path, file_path,
   if expected_status:
     expected_status.tweak(state_path, wc_rev=merged_rev)
   run_and_verify_commit(wc_dir, expected_output, expected_status,
-                        None, None, None, None, None,
-                        file_path)
+                        None, file_path)
 
   # Backdate the file.
-  output, errput = main.run_svn(None, "up", "-r", str(prev_rev),
-                                "--username", main.wc_author,
-                                "--password", main.wc_passwd,
-                                 file_path)
+  exit_code, output, errput = main.run_svn(None, "up", "-r", str(prev_rev),
+                                           file_path)
   if expected_status:
     expected_status.tweak(state_path, wc_rev=prev_rev)
 
@@ -1170,10 +1434,9 @@ def inject_conflict_into_wc(sbox, state_path, file_path,
                                       expected_disk, expected_status,
                                       conflicting_contents, contents,
                                       merged_rev)
-  output, errput = main.run_svn(None, "up", "-r", str(merged_rev),
-                                "--username", main.wc_author,
-                                "--password", main.wc_passwd,
-                                sbox.repo_url + "/" + state_path, file_path)
+  exit_code, output, errput = main.run_svn(None, "up", "-r", str(merged_rev),
+                                           sbox.repo_url + "/" + state_path,
+                                           file_path)
   if expected_status:
     expected_status.tweak(state_path, wc_rev=merged_rev)
 
