@@ -24,6 +24,7 @@ import copy    # for deepcopy()
 import time    # for time()
 import traceback # for print_exc()
 import threading
+import Queue
 
 import getopt
 try:
@@ -88,20 +89,24 @@ if sys.platform == 'win32':
   windows = True
   file_scheme_prefix = 'file:///'
   _exe = '.exe'
+  _bat = '.bat'
 else:
   windows = False
   file_scheme_prefix = 'file://'
   _exe = ''
+  _bat = ''
 
-# os.wait() specifics
 try:
-  from os import wait
-  platform_with_os_wait = True
+  from popen2 import Popen3
+  platform_with_popen3_class = True
 except ImportError:
-  platform_with_os_wait = False
+  platform_with_popen3_class = False
 
 # The location of our mock svneditor script.
-svneditor_script = os.path.join(sys.path[0], 'svneditor.py')
+if sys.platform == 'win32':
+  svneditor_script = os.path.join(sys.path[0], 'svneditor.bat')
+else:
+  svneditor_script = os.path.join(sys.path[0], 'svneditor.py')
 
 # Username and password used by the working copies
 wc_author = 'jrandom'
@@ -125,8 +130,10 @@ svnadmin_binary = os.path.abspath('../../svnadmin/svnadmin' + _exe)
 svnlook_binary = os.path.abspath('../../svnlook/svnlook' + _exe)
 svnsync_binary = os.path.abspath('../../svnsync/svnsync' + _exe)
 svnversion_binary = os.path.abspath('../../svnversion/svnversion' + _exe)
+svndumpfilter_binary = os.path.abspath('../../svndumpfilter/svndumpfilter' + \
+                                       _exe)
 
-# Global variable indicating if we want verbose output, that is, 
+# Global variable indicating if we want verbose output, that is,
 # details of what commands each test does as it does them.  This is
 # incompatible with quiet_mode.
 verbose_mode = False
@@ -174,7 +181,7 @@ fs_type = None
 work_dir = "svn-test-work"
 
 # Constant for the merge info property.
-SVN_PROP_MERGE_INFO = "svn:mergeinfo"
+SVN_PROP_MERGEINFO = "svn:mergeinfo"
 
 # Where we want all the repositories and working copies to live.
 # Each test will have its own!
@@ -225,6 +232,41 @@ greek_state = wc.State('', {
 
 ######################################################################
 # Utilities shared by the tests
+def wrap_ex(func):
+  "Wrap a function, catch, print and ignore exceptions"
+  def w(*args, **kwds):
+    try:
+      return func(*args, **kwds)
+    except Failure, ex:
+      if ex.__class__ != Failure or ex.args:
+        ex_args = str(ex)
+        if ex_args:
+          print 'EXCEPTION: %s: %s' % (ex.__class__.__name__, ex_args)
+        else:
+          print 'EXCEPTION:', ex.__class__.__name__
+  return w
+
+def setup_development_mode():
+  "Wraps functions in module actions"
+  l = [ 'run_and_verify_svn',
+        'run_and_verify_svnversion',
+        'run_and_verify_load',
+        'run_and_verify_dump',
+        'run_and_verify_checkout',
+        'run_and_verify_export',
+        'run_and_verify_update',
+        'run_and_verify_merge',
+        'run_and_verify_merge2',
+        'run_and_verify_switch',
+        'run_and_verify_commit',
+        'run_and_verify_unquiet_status',
+        'run_and_verify_status',
+        'run_and_verify_diff_summarize',
+        'run_and_verify_diff_summarize_xml',
+        'run_and_validate_lock']
+  
+  for func in l:
+    setattr(actions, func, wrap_ex(getattr(actions, func)))
 
 def get_admin_name():
   "Return name of SVN administrative subdirectory."
@@ -262,9 +304,10 @@ def get_svnserve_conf_file_path(repo_dir):
 
   return os.path.join(repo_dir, "conf", "svnserve.conf")
 
-# Run any binary, logging the command line (TODO: and return code)
+# Run any binary, logging the command line and return code
 def run_command(command, error_expected, binary_mode=0, *varargs):
-  """Run COMMAND with VARARGS; return stdout, stderr as lists of lines.
+  """Run COMMAND with VARARGS; return exit code as int; stdout, stderr
+  as lists of lines.
   If ERROR_EXPECTED is None, any stderr also will be printed."""
 
   return run_command_stdin(command, error_expected, binary_mode,
@@ -296,12 +339,52 @@ def _quote_arg(arg):
       arg = arg.replace('$', '\$')
     return '"%s"' % (arg,)
 
+def open_pipe(command, mode):
+  """Opens a popen3 pipe to COMMAND in MODE.
+
+  Returns (infile, outfile, errfile, waiter); waiter
+  should be passed to wait_on_pipe."""
+  if platform_with_popen3_class:
+    kid = Popen3(command, True)
+    return kid.tochild, kid.fromchild, kid.childerr, (kid, command)
+  else:
+    inf, outf, errf = os.popen3(command, mode)
+    return inf, outf, errf, None
+
+def wait_on_pipe(waiter, stdout_lines, stderr_lines):
+  """Waits for KID (opened with open_pipe) to finish, dying
+  if it does.  Uses STDOUT_LINES and STDERR_LINES for error message
+  if kid fails.  Returns kid's exit code."""
+  if waiter is None:
+    return
+
+  kid, command = waiter
+
+  wait_code = kid.wait()
+
+  if os.WIFSIGNALED(wait_code):
+    exit_signal = os.WTERMSIG(wait_code)
+    if stdout_lines is not None:
+      sys.stdout.write("".join(stdout_lines))
+    if stderr_lines is not None:
+      sys.stderr.write("".join(stderr_lines))
+    if verbose_mode:
+      # show the whole path to make it easier to start a debugger
+      sys.stderr.write("CMD: %s terminated by signal %d\n"
+                       % (command, exit_signal))
+    raise SVNProcessTerminatedBySignal
+  else:
+    exit_code = os.WEXITSTATUS(wait_code)
+    if exit_code and verbose_mode:
+      sys.stderr.write("CMD: %s exited with %d\n" % (command, exit_code))
+    return exit_code
+
 # Run any binary, supplying input text, logging the command line
 def spawn_process(command, binary_mode=0,stdin_lines=None, *varargs):
   args = ' '.join(map(_quote_arg, varargs))
 
   # Log the command line
-  if verbose_mode:
+  if verbose_mode and not command.endswith('.py'):
     print 'CMD:', os.path.basename(command) + ' ' + args,
 
   if binary_mode:
@@ -309,7 +392,7 @@ def spawn_process(command, binary_mode=0,stdin_lines=None, *varargs):
   else:
     mode = 't'
 
-  infile, outfile, errfile = os.popen3(command + ' ' + args, mode)
+  infile, outfile, errfile, kid = open_pipe(command + ' ' + args, mode)
 
   if stdin_lines:
     map(infile.write, stdin_lines)
@@ -322,24 +405,7 @@ def spawn_process(command, binary_mode=0,stdin_lines=None, *varargs):
   outfile.close()
   errfile.close()
 
-  exit_code = 0
-
-  if platform_with_os_wait:
-    pid, wait_code = os.wait()
-
-    if os.WIFSIGNALED(wait_code):
-      exit_signal = os.WTERMSIG(wait_code)
-      sys.stdout.write("".join(stdout_lines))
-      sys.stderr.write("".join(stderr_lines))
-      if verbose_mode:
-        # show the whole path to make it easier to start a debugger
-        sys.stderr.write("CMD: %s terminated by signal %d\n"
-                         % (command, exit_signal))
-      raise SVNProcessTerminatedBySignal
-    else:
-      exit_code = os.WEXITSTATUS(wait_code)
-      if exit_code and verbose_mode:
-        sys.stderr.write("CMD: %s exited with %d\n" % (command, exit_code))
+  exit_code = wait_on_pipe(kid, stdout_lines, stderr_lines)
 
   return exit_code, stdout_lines, stderr_lines
 
@@ -350,7 +416,7 @@ def run_command_stdin(command, error_expected, binary_mode=0,
   should not be very large, as if the program outputs more than the OS
   is willing to buffer, this will deadlock, with both Python and
   COMMAND waiting to write to each other for ever.
-  Return stdout, stderr as lists of lines.
+  Return exit code as int; stdout, stderr as lists of lines.
   If ERROR_EXPECTED is None, any stderr also will be printed."""
 
   if verbose_mode:
@@ -369,7 +435,7 @@ def run_command_stdin(command, error_expected, binary_mode=0,
     map(sys.stdout.write, stderr_lines)
     raise Failure
 
-  return stdout_lines, stderr_lines
+  return exit_code, stdout_lines, stderr_lines
 
 def create_config_dir(cfgdir, config_contents=None, server_contents=None):
   "Create config directories and files"
@@ -410,30 +476,45 @@ def _with_config_dir(args):
   else:
     return args + ('--config-dir', default_config_dir)
 
+def _with_auth(args):
+  assert '--password' not in args
+  args = args + ('--password', wc_passwd,
+                 '--no-auth-cache' )
+  if '--username' in args:
+    return args
+  else:
+    return args + ('--username', wc_author )
+
 # For running subversion and returning the output
 def run_svn(error_expected, *varargs):
-  """Run svn with VARARGS; return stdout, stderr as lists of lines.
+  """Run svn with VARARGS; return exit code as int; stdout, stderr as
+  lists of lines.
   If ERROR_EXPECTED is None, any stderr also will be printed.  If
   you're just checking that something does/doesn't come out of
   stdout/stderr, you might want to use actions.run_and_verify_svn()."""
-  return run_command(svn_binary, error_expected, 0, *(_with_config_dir(varargs)))
+  return run_command(svn_binary, error_expected, 0,
+                     *(_with_auth(_with_config_dir(varargs))))
 
 # For running svnadmin.  Ignores the output.
 def run_svnadmin(*varargs):
-  "Run svnadmin with VARARGS, returns stdout, stderr as list of lines."
+  """Run svnadmin with VARARGS, returns exit code as int; stdout, stderr as
+  list of lines."""
   return run_command(svnadmin_binary, 1, 0, *varargs)
 
 # For running svnlook.  Ignores the output.
 def run_svnlook(*varargs):
-  "Run svnlook with VARARGS, returns stdout, stderr as list of lines."
+  """Run svnlook with VARARGS, returns exit code as int; stdout, stderr as
+  list of lines."""
   return run_command(svnlook_binary, 1, 0, *varargs)
 
 def run_svnsync(*varargs):
-  "Run svnsync with VARARGS, returns stdout, stderr as list of lines."
+  """Run svnsync with VARARGS, returns exit code as int; stdout, stderr as
+  list of lines."""
   return run_command(svnsync_binary, 1, 0, *(_with_config_dir(varargs)))
 
 def run_svnversion(*varargs):
-  "Run svnversion with VARARGS, returns stdout, stderr as list of lines."
+  """Run svnversion with VARARGS, returns exit code as int; stdout, stderr
+  as list of lines."""
   return run_command(svnversion_binary, 1, 0, *varargs)
 
 # Chmod recursively on a whole subtree
@@ -520,7 +601,8 @@ def create_repos(path):
     opts += ("--pre-1.5-compatible",)
   if fs_type is not None:
     opts += ("--fs-type=" + fs_type,)
-  stdout, stderr = run_command(svnadmin_binary, 1, 0, "create", path, *opts)
+  exit_code, stdout, stderr = run_command(svnadmin_binary, 1, 0, "create",
+                                          path, *opts)
 
   # Skip tests if we can't create the repository.
   if stderr:
@@ -560,8 +642,11 @@ def copy_repos(src_path, dst_path, head_revision, ignore_uuid = 1):
     print 'CMD:', os.path.basename(svnadmin_binary) + dump_args, \
           '|', os.path.basename(svnadmin_binary) + load_args,
   start = time.time()
-  dump_in, dump_out, dump_err = os.popen3(svnadmin_binary + dump_args, 'b')
-  load_in, load_out, load_err = os.popen3(svnadmin_binary + load_args, 'b')
+
+  dump_in, dump_out, dump_err, dump_kid = \
+           open_pipe(svnadmin_binary + dump_args, 'b')
+  load_in, load_out, load_err, load_kid = \
+           open_pipe(svnadmin_binary + load_args, 'b')
   stop = time.time()
   if verbose_mode:
     print '<TIME = %.6f>' % (stop - start)
@@ -580,6 +665,9 @@ def copy_repos(src_path, dst_path, head_revision, ignore_uuid = 1):
   dump_err.close()
   load_out.close()
   load_err.close()
+  # Wait on the pipes; ignore return code.
+  wait_on_pipe(dump_kid, None, dump_lines)
+  wait_on_pipe(load_kid, load_lines, None)
 
   dump_re = re.compile(r'^\* Dumped revision (\d+)\.\r?$')
   expect_revision = 0
@@ -647,6 +735,14 @@ def write_restrictive_svnserve_conf(repo_dir, anon_access="none"):
     fp.write("password-db = passwd\n")
   fp.close()
 
+# Warning: because mod_dav_svn uses one shared authz file for all
+# repositories, you *cannot* use write_authz_file in any test that
+# might be run in parallel.
+#
+# write_authz_file can *only* be used in test suites which disable
+# parallel execution at the bottom like so
+#   if __name__ == '__main__':
+#     svntest.main.run_tests(test_list, serial_only = True)
 def write_authz_file(sbox, rules, sections=None):
   """Write an authz file to SBOX, appropriate for the RA method used,
 with authorizations rules RULES mapping paths to strings containing
@@ -654,8 +750,16 @@ the rules. You can add sections SECTIONS (ex. groups, aliases...) with
 an appropriate list of mappings.
 """
   fp = open(sbox.authz_file, 'w')
+
+  # When the sandbox repository is read only it's name will be different from
+  # the repository name.
+  repo_name = sbox.repo_dir
+  while repo_name[-1] == '/':
+    repo_name = repo_name[:-1]
+  repo_name = os.path.basename(repo_name)
+
   if sbox.repo_url.startswith("http"):
-    prefix = sbox.name + ":"
+    prefix = repo_name + ":"
   else:
     prefix = ""
   if sections:
@@ -672,11 +776,13 @@ def use_editor(func):
   os.environ['SVNTEST_EDITOR_FUNC'] = func
 
 
-def merge_notify_line(revstart=None, revend=None):
+def merge_notify_line(revstart=None, revend=None, same_URL=True):
   """Return an expected output line that describes the beginning of a
   merge operation on revisions REVSTART through REVEND.  Omit both
   REVSTART and REVEND for the case where the left and right sides of
   the merge are from different URLs."""
+  if not same_URL:
+    return "--- Merging differences between repository URLs into '.+':\n"
   if revend is None:
     if revstart is None:
       # The left and right sides of the merge are from different URLs.
@@ -736,6 +842,22 @@ def server_has_revprop_commit():
   _check_command_line_parsed()
   return server_minor_version >= 5
 
+def server_sends_copyfrom_on_update():
+  _check_command_line_parsed()
+  return server_minor_version >= 5
+
+def server_authz_has_aliases():
+  _check_command_line_parsed()
+  return server_minor_version >= 5
+
+def server_gets_client_capabilities():
+  _check_command_line_parsed()
+  return server_minor_version >= 5
+
+def server_has_partial_replay():
+  _check_command_line_parsed()
+  return server_minor_version >= 5
+
 
 ######################################################################
 # Sandbox handling
@@ -749,14 +871,21 @@ class Sandbox:
   def __init__(self, module, idx):
     self._set_name("%s-%d" % (module, idx))
 
-  def _set_name(self, name):
+  def _set_name(self, name, read_only = False):
     """A convenience method for renaming a sandbox, useful when
     working with multiple repositories in the same unit test."""
-    self.name = name
+    if not name is None:
+      self.name = name
+    self.read_only = read_only
     self.wc_dir = os.path.join(general_wc_dir, self.name)
-    self.repo_dir = os.path.join(general_repo_dir, self.name)
-    self.repo_url = test_area_url + '/' + self.repo_dir
-
+    if not read_only:
+      self.repo_dir = os.path.join(general_repo_dir, self.name)
+      self.repo_url = test_area_url + '/' + self.repo_dir
+    else:
+      self.repo_dir = pristine_dir
+      self.repo_url = pristine_url
+      
+    ### TODO: Move this into to the build() method
     # For dav tests we need a single authz file which must be present,
     # so we recreate it each time a sandbox is created with some default
     # contents.
@@ -777,31 +906,39 @@ class Sandbox:
       self.repo_url = self.repo_url.replace('\\', '/')
     self.test_paths = [self.wc_dir, self.repo_dir]
 
-  def clone_dependent(self):
+  def clone_dependent(self, copy_wc=False):
     """A convenience method for creating a near-duplicate of this
     sandbox, useful when working with multiple repositories in the
-    same unit test.  Any necessary cleanup operations are triggered
-    by cleanup of the original sandbox."""
+    same unit test.  If COPY_WC is true, make an exact copy of this
+    sandbox's working copy at the new sandbox's working copy
+    directory.  Any necessary cleanup operations are triggered by
+    cleanup of the original sandbox."""
+
     if not self.dependents:
       self.dependents = []
-    self.dependents.append(copy.deepcopy(self))
-    self.dependents[-1]._set_name("%s-%d" % (self.name, len(self.dependents)))
-    return self.dependents[-1]
+    clone = copy.deepcopy(self)
+    self.dependents.append(clone)
+    clone._set_name("%s-%d" % (self.name, len(self.dependents)))
+    if copy_wc:
+      self.add_test_path(clone.wc_dir)
+      shutil.copytree(self.wc_dir, clone.wc_dir, symlinks=True)
+    return clone
 
-  def build(self, name = None, create_wc = True):
-    if name != None:
-      self._set_name(name)
-    if actions.make_repo_and_wc(self, create_wc):
+  def build(self, name = None, create_wc = True, read_only = False):
+    self._set_name(name, read_only)
+    if actions.make_repo_and_wc(self, create_wc, read_only):
       raise Failure("Could not build repository and sandbox '%s'" % self.name)
 
-  def add_test_path(self, path, remove=1):
+  def add_test_path(self, path, remove=True):
     self.test_paths.append(path)
     if remove:
       safe_rmtree(path)
 
   def add_repo_path(self, suffix, remove=1):
-    path = self.repo_dir + '.' + suffix
-    url  = self.repo_url + '.' + suffix
+    path = os.path.join(general_repo_dir, self.name)  + '.' + suffix
+    url  = test_area_url + '/' + path
+    if windows:
+      url = url.replace('\\', '/')
     self.add_test_path(path, remove)
     return path, url
 
@@ -816,8 +953,10 @@ class Sandbox:
       # Recursively cleanup any dependent sandboxes.
       for sbox in self.dependents:
         sbox.cleanup_test_paths()
+    # cleanup all test specific working copies and repositories
     for path in self.test_paths:
-      _cleanup_test_path(path)
+      if not path is pristine_dir:
+        _cleanup_test_path(path)
 
 
 _deferred_test_paths = []
@@ -841,23 +980,29 @@ def _cleanup_test_path(path, retrying=None):
       print "WARNING: cleanup failed, will try again later"
     _deferred_test_paths.append(path)
 
-class SpawnTest(threading.Thread):
-  """Encapsulate a single test case, run it in a separate child process.
-  Instead of waiting till the process is finished, add this class to a
-  list of active tests for follow up in the parent process."""
-  def __init__(self, index, tests = None):
+class TestSpawningThread(threading.Thread):
+  """A thread that runs test cases in their own processes.
+  Receives test numbers to run from the queue, and saves results into
+  the results field."""
+  def __init__(self, queue):
     threading.Thread.__init__(self)
-    self.index = index
-    self.tests = tests
-    self.result = None
-    self.stdout_lines = None
-    self.stderr_lines = None
+    self.queue = queue
+    self.results = []
 
   def run(self):
+    while True:
+      try:
+        next_index = self.queue.get_nowait()
+      except Queue.Empty:
+        return
+
+      self.run_one(next_index)
+
+  def run_one(self, index):
     command = sys.argv[0]
 
     args = []
-    args.append(str(self.index))
+    args.append(str(index))
     args.append('-c')
     # add some startup arguments from this process
     if fs_type:
@@ -870,15 +1015,19 @@ class SpawnTest(threading.Thread):
       args.append('--cleanup')
     if enable_sasl:
       args.append('--enable-sasl')
+    if http_library:
+      args.append('--http-library=' + http_library)
+    if server_minor_version:
+      args.append('--server-minor-version=' + str(server_minor_version))
 
-    self.result, self.stdout_lines, self.stderr_lines =\
-                                         spawn_process(command, 1, None, *args)
-    # don't trust the exitcode, will not be correct on Windows
+    result, stdout_lines, stderr_lines = spawn_process(command, 1, None, *args)
+    # "result" will be None on platforms without Popen3 (e.g. Windows)
     if filter(lambda x: x.startswith('FAIL: ') or x.startswith('XPASS: '),
-              self.stdout_lines):
-      self.result = 1
-    self.tests.append(self)
+              stdout_lines):
+      result = 1
+    self.results.append((index, result, stdout_lines, stderr_lines))
     sys.stdout.write('.')
+    sys.stdout.flush()
 
 class TestRunner:
   """Encapsulate a single test case (predicate), including logic for
@@ -1014,34 +1163,35 @@ def _internal_run_tests(test_list, testnums, parallel):
       if run_one_test(testnum, test_list) == 1:
           exit_code = 1
   else:
-    for testnum in testnums:
-      # wait till there's a free spot.
-      while tests_started - len(finished_tests) > parallel:
-        time.sleep(0.2)
-      run_one_test(testnum, test_list, parallel, finished_tests)
-      tests_started += 1
+    number_queue = Queue.Queue()
+    for num in testnums:
+      number_queue.put(num)
 
-    # wait for all tests to finish
-    while len(finished_tests) < len(testnums):
-      time.sleep(0.2)
+    threads = [ TestSpawningThread(number_queue) for i in range(parallel) ]
+    for t in threads:
+      t.start()
 
-    # Sort test results list by test nr.
-    deco = [(test.index, test) for test in finished_tests]
-    deco.sort()
-    finished_tests = [test for (ti, test) in deco]
+    for t in threads:
+      t.join()
+
+    # list of (index, result, stdout, stderr)
+    results = []
+    for t in threads:
+      results += t.results
+    results.sort()
 
     # terminate the line of dots
     print
 
     # all tests are finished, find out the result and print the logs.
-    for test in finished_tests:
-      if test.stdout_lines:
-        for line in test.stdout_lines:
+    for (index, result, stdout_lines, stderr_lines) in results:
+      if stdout_lines:
+        for line in stdout_lines:
           sys.stdout.write(line)
-      if test.stderr_lines:
-        for line in test.stderr_lines:
+      if stderr_lines:
+        for line in stderr_lines:
           sys.stdout.write(line)
-      if test.result == 1:
+      if result == 1:
         exit_code = 1
 
   _cleanup_deferred_test_paths()
@@ -1050,7 +1200,8 @@ def _internal_run_tests(test_list, testnums, parallel):
 
 def usage():
   prog_name = os.path.basename(sys.argv[0])
-  print "%s [--url] [--fs-type] [--verbose|--quiet] \\" % prog_name
+  print "%s [--url] [--fs-type] [--verbose|--quiet] [--parallel] \\" % \
+        prog_name
   print "%s [--enable-sasl] [--cleanup] [--bin] [<test> ...]" \
       % (" " * len(prog_name))
   print "%s " % (" " * len(prog_name))
@@ -1069,6 +1220,14 @@ def usage():
   print " --enable-sasl   Whether to enable SASL authentication"
   print " --parallel      Run the tests in parallel"
   print " --bin           Use the svn binaries installed in this path"
+  print " --use-jsvn      Use the jsvn (SVNKit based) binaries. Can be\n" \
+        "                 combined with --bin to point to a specific path"
+  print " --development   Test development mode: provides more detailed test\n"\
+        "                 output and ignores all exceptions in the \n"  \
+        "                 run_and_verify* functions. This option is only \n" \
+        "                 useful during test development!"
+  print " --server-minor-version  Set the minor version for the server.\n" \
+        "                 Supports version 4 or 5."
   print " --help          This information"
 
 
@@ -1106,11 +1265,14 @@ def run_tests(test_list, serial_only = False):
 
   parallel = 0
   svn_bin = None
+  use_jsvn = False
+
   try:
     opts, args = my_getopt(sys.argv[1:], 'vqhpc',
                            ['url=', 'fs-type=', 'verbose', 'quiet', 'cleanup',
                             'list', 'enable-sasl', 'help', 'parallel',
-                            'bin=', 'http-library=', 'server-minor-version='])
+                            'bin=', 'http-library=', 'server-minor-version=', 
+                            'use-jsvn', 'development'])
   except getopt.GetoptError, e:
     print "ERROR: %s\n" % e
     usage()
@@ -1174,6 +1336,12 @@ def run_tests(test_list, serial_only = False):
         print "ERROR: test harness only supports server minor version 4 or 5"
         sys.exit(1)
 
+    elif opt == '--use-jsvn':
+      use_jsvn = True
+
+    elif opt == '--development':
+      setup_development_mode()
+
   if test_area_url[-1:] == '/': # Normalize url to have no trailing slash
     test_area_url = test_area_url[:-1]
 
@@ -1186,12 +1354,22 @@ def run_tests(test_list, serial_only = False):
   if windows:
     pristine_url = pristine_url.replace('\\', '/')
 
-  if not svn_bin is None:
-    svn_binary = os.path.join(svn_bin, 'svn' + _exe)
-    svnadmin_binary = os.path.join(svn_bin, 'svnadmin' + _exe)
-    svnlook_binary = os.path.join(svn_bin, 'svnlook' + _exe)
-    svnsync_binary = os.path.join(svn_bin, 'svnsync' + _exe)
-    svnversion_binary = os.path.join(svn_bin, 'svnversion' + _exe)
+  if use_jsvn:
+    if svn_bin is None: 
+      svn_bin = ''
+    svn_binary = os.path.join(svn_bin, 'jsvn' + _bat)
+    svnadmin_binary = os.path.join(svn_bin, 'jsvnadmin' + _bat)
+    svnlook_binary = os.path.join(svn_bin, 'jsvnlook' + _bat)
+    svnsync_binary = os.path.join(svn_bin, 'jsvnsync' + _bat)
+    svnversion_binary = os.path.join(svn_bin, 'jsvnversion' + _bat)
+    use_jsvn = False
+  else:
+    if svn_bin:
+      svn_binary = os.path.join(svn_bin, 'svn' + _exe)
+      svnadmin_binary = os.path.join(svn_bin, 'svnadmin' + _exe)
+      svnlook_binary = os.path.join(svn_bin, 'svnlook' + _exe)
+      svnsync_binary = os.path.join(svn_bin, 'svnsync' + _exe)
+      svnversion_binary = os.path.join(svn_bin, 'svnversion' + _exe)
 
   command_line_parsed = True
 
@@ -1200,7 +1378,7 @@ def run_tests(test_list, serial_only = False):
   # Cleanup: if a previous run crashed or interrupted the python
   # interpreter, then `temp_dir' was never removed.  This can cause wonkiness.
   if not is_child_process:
-    safe_rmtree(temp_dir)
+    safe_rmtree(temp_dir, 1)
 
   if not testnums:
     # If no test numbers were listed explicitly, include all of them:
@@ -1232,7 +1410,7 @@ def run_tests(test_list, serial_only = False):
   # Remove all scratchwork: the 'pristine' repository, greek tree, etc.
   # This ensures that an 'import' will happen the next time we run.
   if not is_child_process:
-    safe_rmtree(temp_dir)
+    safe_rmtree(temp_dir, 1)
 
   # Cleanup after ourselves.
   _cleanup_deferred_test_paths()
