@@ -321,7 +321,7 @@ def get_props(path):
   # respecting the black-box paradigm.
 
   props = {}
-  output, errput = main.run_svn(1, "proplist", path, "--verbose")
+  exit_code, output, errput = main.run_svn(1, "proplist", path, "--verbose")
 
   first_value = 0
   for line in output:
@@ -340,7 +340,24 @@ def get_props(path):
         # we knew this was a multiline property.
         props[name] = props[name] + "\n"
         first_value = 0
-      props[name] = props[name] + line
+      # Keep the line endings consistent with what was done to the first
+      # line by stripping whitespace and then appending a newline.  This
+      # prevents multiline props on Windows that must be stored as UTF8/LF
+      # in the repository (e.g. svn:mergeinfo), say like this:
+      #
+      #   "propname : propvalLine1<LF>propvalLine2<LF>propvalLine3"
+      #
+      # but that print to stdout like this:
+      #
+      #   Properties on 'somepath':<CR><LF>
+      #     propname : propvalLine1<CR><CR><LF>
+      #   propvalLine1<CR><CR><LF>
+      #   propvalLine1<CR><LF>
+      #
+      # from looking like this in the returned PROPS hash:
+      #
+      #   "propname" --> "propvalLine1<LF>propvalLine2<CR><LF>propvalLine3<LF>"
+      props[name] = props[name] + line.strip() + "\n"
 
   return props
 
@@ -407,17 +424,12 @@ def get_child(node, name):
   return None
 
 
-# Helpers for compare_trees
-def default_singleton_handler_a(a, baton):
-  "Printing SVNTreeNode A's name, then raise SVNTreeUnequal."
-  print "Couldn't find node '%s' in expected tree" % a.name
-  a.pprint()
-  raise SVNTreeUnequal
-
-def default_singleton_handler_b(b, baton):
-  "Printing SVNTreeNode B's name, then raise SVNTreeUnequal."
-  print "Couldn't find node '%s' in actual tree" % b.name
-  b.pprint()
+# Helper for compare_trees
+def default_singleton_handler(node, description):
+  """Print SVNTreeNode NODE's name, describing it with the string
+  DESCRIPTION, then raise SVNTreeUnequal."""
+  print "Couldn't find node '%s' in %s tree" % (node.name, description)
+  node.pprint()
   raise SVNTreeUnequal
 
 # A test helper function implementing the singleton_handler_a API.
@@ -445,7 +457,8 @@ def detect_conflict_files(node, extra_files):
 
 # Main tree comparison routine!
 
-def compare_trees(a, b,
+def compare_trees(label,
+                  a, b,
                   singleton_handler_a = None,
                   a_baton = None,
                   singleton_handler_b = None,
@@ -453,7 +466,10 @@ def compare_trees(a, b,
   """Compare SVNTreeNodes A (actual) and B (expected), expressing
   differences using FUNC_A and FUNC_B.  FUNC_A and FUNC_B are
   functions of two arguments (a SVNTreeNode and a context baton), and
-  may raise exception SVNTreeUnequal.  Their return value is ignored.
+  may raise exception SVNTreeUnequal, in which case they use the
+  string LABEL to describe the error (their return value is ignored).
+  LABEL is typically "output", "disk", "status", or some other word
+  that labels the trees being compared.
 
   If A and B are both files, then return if their contents,
   properties, and names are all the same; else raise a SVNTreeUnequal.
@@ -466,7 +482,8 @@ def compare_trees(a, b,
   def display_nodes(a, b):
     'Display two nodes, expected and actual.'
     print "============================================================="
-    print "Expected '%s' and actual '%s' are different!" % (b.name, a.name)
+    print "Expected '%s' and actual '%s' in %s tree are different!" \
+          % (b.name, a.name, label)
     print "============================================================="
     print "EXPECTED NODE TO BE:"
     print "============================================================="
@@ -478,9 +495,11 @@ def compare_trees(a, b,
 
   # Setup singleton handlers
   if (singleton_handler_a is None):
-    singleton_handler_a = default_singleton_handler_a
+    singleton_handler_a = default_singleton_handler
+    a_baton = "expected " + label
   if (singleton_handler_b is None):
-    singleton_handler_b = default_singleton_handler_b
+    singleton_handler_b = default_singleton_handler
+    b_baton = "actual " + label
 
   try:
     # A and B are both files.
@@ -510,7 +529,7 @@ def compare_trees(a, b,
         b_child = get_child(b, a_child.name)
         if b_child:
           accounted_for.append(b_child)
-          compare_trees(a_child, b_child,
+          compare_trees(label, a_child, b_child,
                         singleton_handler_a, a_baton,
                         singleton_handler_b, b_baton)
         else:
@@ -604,7 +623,7 @@ def build_tree_from_checkout(lines, include_skipped=1):
   "Return a tree derived by parsing the output LINES from 'co' or 'up'."
 
   root = SVNTreeNode(root_node_name)
-  rm1 = re.compile ('^([MAGCUDE_ ][MAGCUDE_ ])([B ])\s+(.+)')
+  rm1 = re.compile ('^([RMAGCUDE_ ][MAGCUDE_ ])([B ])\s+(.+)')
   if include_skipped:
     rm2 = re.compile ('^(Restored|Skipped)\s+\'(.+)\'')
   else:
